@@ -1,6 +1,5 @@
 /***************************************************************************
  *   Copyright (C) 2007 Ryan Schultz, PCSX-df Team, PCSX team              *
- *   schultz.ryan@gmail.com, http://rschultz.ath.cx/code.php               *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -15,23 +14,22 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Steet, Fifth Floor, Boston, MA 02111-1307 USA.            *
+ *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
  ***************************************************************************/
 
 /*
-* Miscellaneous functions, including savesates and CD-ROM loading.
+* Miscellaneous functions, including savestates and CD-ROM loading.
 */
 
 #include "misc.h"
 #include "cdrom.h"
 #include "mdec.h"
-
-int Log = 0;
+#include "ppf.h"
 
 char CdromId[10] = "";
 char CdromLabel[33] = "";
 
-/* PSX Executable types */
+// PSX Executable types
 #define PSX_EXE     1
 #define CPE_EXE     2
 #define COFF_EXE    3
@@ -53,28 +51,25 @@ struct iso_directory_record {
 	char name			[1];
 };
 
-#define btoi(b)		((b)/16*10 + (b)%16)		/* BCD to u_char */
-#define itob(i)		((i)/10*16 + (i)%10)		/* u_char to BCD */
-
 void mmssdd( char *b, char *p )
- {
+{
 	int m, s, d;
 #if defined(__BIGENDIAN__)
-	int block = (b[0]&0xff) | ((b[1]&0xff)<<8) | ((b[2]&0xff)<<16) | (b[3]<<24);
+	int block = (b[0] & 0xff) | ((b[1] & 0xff) << 8) | ((b[2] & 0xff) << 16) | (b[3] << 24);
 #else
 	int block = *((int*)b);
 #endif
-	
+
 	block += 150;
-	m = block / 4500;			// minuten
-	block = block - m * 4500;	// minuten rest
-	s = block / 75;				// sekunden
-	d = block - s * 75;			// sekunden rest
-	
-	m = ( ( m / 10 ) << 4 ) | m % 10;
-	s = ( ( s / 10 ) << 4 ) | s % 10;
-	d = ( ( d / 10 ) << 4 ) | d % 10;	
-	
+	m = block / 4500;			// minutes
+	block = block - m * 4500;	// minutes rest
+	s = block / 75;				// seconds
+	d = block - s * 75;			// seconds rest
+
+	m = ((m / 10) << 4) | m % 10;
+	s = ((s / 10) << 4) | s % 10;
+	d = ((d / 10) << 4) | d % 10;	
+
 	p[0] = m;
 	p[1] = s;
 	p[2] = d;
@@ -95,15 +90,16 @@ void mmssdd( char *b, char *p )
 
 #define READTRACK() \
 	if (CDR_readTrack(time) == -1) return -1; \
-	buf = CDR_getBuffer(); if (buf == NULL) return -1;
+	buf = CDR_getBuffer(); \
+	if (buf == NULL) return -1; else CheckPPFCache(buf, time[0], time[1], time[2]);
 
 #define READDIR(_dir) \
 	READTRACK(); \
-	memcpy(_dir, buf+12, 2048); \
+	memcpy(_dir, buf + 12, 2048); \
  \
 	incTime(); \
 	READTRACK(); \
-	memcpy(_dir+2048, buf+12, 2048);
+	memcpy(_dir + 2048, buf + 12, 2048);
 
 int GetCdromFile(u8 *mdir, u8 *time, s8 *filename) {
 	struct iso_directory_record *dir;
@@ -112,8 +108,8 @@ int GetCdromFile(u8 *mdir, u8 *time, s8 *filename) {
 	int i;
 
 	// only try to scan if a filename is given
-	if(!strlen(filename)) return -1;
-	
+	if (!strlen(filename)) return -1;
+
 	i = 0;
 	while (i < 4096) {
 		dir = (struct iso_directory_record*) &mdir[i];
@@ -123,19 +119,19 @@ int GetCdromFile(u8 *mdir, u8 *time, s8 *filename) {
 		i += dir->length[0];
 
 		if (dir->flags[0] & 0x2) { // it's a dir
-			if (!strnicmp((char*)&dir->name[0], filename, dir->name_len[0])) {
+			if (!strnicmp((char *)&dir->name[0], filename, dir->name_len[0])) {
 				if (filename[dir->name_len[0]] != '\\') continue;
-				
-				filename+= dir->name_len[0] + 1;
 
-				mmssdd(dir->extent, (char*)time);
+				filename += dir->name_len[0] + 1;
+
+				mmssdd(dir->extent, (char *)time);
 				READDIR(ddir);
 				i = 0;
 				mdir = ddir;
 			}
 		} else {
-			if (!strnicmp((char*)&dir->name[0], filename, strlen(filename))) {
-				mmssdd(dir->extent, (char*)time);
+			if (!strnicmp((char *)&dir->name[0], filename, strlen(filename))) {
+				mmssdd(dir->extent, (char *)time);
 				break;
 			}
 		}
@@ -146,12 +142,12 @@ int GetCdromFile(u8 *mdir, u8 *time, s8 *filename) {
 int LoadCdrom() {
 	EXE_HEADER tmpHead;
 	struct iso_directory_record *dir;
-	u8 time[4],*buf;
+	u8 time[4], *buf;
 	u8 mdir[4096];
 	s8 exename[256];
 
 	if (!Config.HLE) {
-		psxRegs.pc = psxRegs.GPR.n.ra;
+		if (!Config.SlowBoot) psxRegs.pc = psxRegs.GPR.n.ra;
 		return 0;
 	}
 
@@ -177,16 +173,23 @@ int LoadCdrom() {
 		// read the SYSTEM.CNF
 		READTRACK();
 
-		sscanf((char*)buf+12, "BOOT = cdrom:\\%256s", exename);
+		sscanf((char *)buf + 12, "BOOT = cdrom:\\%256s", exename);
 		if (GetCdromFile(mdir, time, exename) == -1) {
-			sscanf((char*)buf+12, "BOOT = cdrom:%256s", exename);
+			sscanf((char *)buf + 12, "BOOT = cdrom:%256s", exename);
 			if (GetCdromFile(mdir, time, exename) == -1) {
-				char *ptr = strstr(buf+12, "cdrom:");
-				if(ptr) {
-					strncpy(exename, ptr, 256);
+				char *ptr = strstr(buf + 12, "cdrom:");
+				if (ptr != NULL) {
+					ptr += 6;
+					while (*ptr == '\\' || *ptr == '/') ptr++;
+					strncpy(exename, ptr, 255);
+					exename[255] = '\0';
+					ptr = exename;
+					while (*ptr != '\0' && *ptr != '\r' && *ptr != '\n') ptr++;
+					*ptr = '\0';
 					if (GetCdromFile(mdir, time, exename) == -1)
 						return -1;
-				}
+				} else
+					return -1;
 			}
 		}
 
@@ -194,8 +197,7 @@ int LoadCdrom() {
 		READTRACK();
 	}
 
-	
-	memcpy(&tmpHead, buf+12, sizeof(EXE_HEADER));
+	memcpy(&tmpHead, buf + 12, sizeof(EXE_HEADER));
 
 	psxRegs.pc = SWAP32(tmpHead.pc0);
 	psxRegs.GPR.n.gp = SWAP32(tmpHead.gp0);
@@ -221,7 +223,7 @@ int LoadCdrom() {
 	return 0;
 }
 
-int LoadCdromFile(char *filename, EXE_HEADER *head) {
+int LoadCdromFile(const char *filename, EXE_HEADER *head) {
 	struct iso_directory_record *dir;
 	u8 time[4],*buf;
 	u8 mdir[4096], exename[256];
@@ -234,7 +236,7 @@ int LoadCdromFile(char *filename, EXE_HEADER *head) {
 	READTRACK();
 
 	// skip head and sub, and go to the root directory record
-	dir = (struct iso_directory_record*) &buf[12+156]; 
+	dir = (struct iso_directory_record *)&buf[12 + 156]; 
 
 	mmssdd(dir->extent, (char*)time);
 
@@ -244,15 +246,19 @@ int LoadCdromFile(char *filename, EXE_HEADER *head) {
 
 	READTRACK();
 
-	memcpy(head, buf+12, sizeof(EXE_HEADER));
+	memcpy(head, buf + 12, sizeof(EXE_HEADER));
 	size = head->t_size;
 	addr = head->t_addr;
 
+	// Cache clear/invalidate dynarec/int. Fixes startup of Casper/X-Files and possibly others.
+	psxCpu->Clear(addr, size / 4);
+	psxRegs.ICache_valid = FALSE;
+
 	while (size) {
 		incTime();
 		READTRACK();
 
-		memcpy((void *)PSXM(addr), buf+12, 2048);
+		memcpy((void *)PSXM(addr), buf + 12, 2048);
 
 		size -= 2048;
 		addr += 2048;
@@ -266,7 +272,9 @@ int CheckCdrom() {
 	unsigned char time[4], *buf;
 	unsigned char mdir[4096];
 	char exename[256];
-	int i, c;
+	int i, len, c;
+
+	FreePPFCache();
 
 	time[0] = itob(0);
 	time[1] = itob(2);
@@ -274,8 +282,9 @@ int CheckCdrom() {
 
 	READTRACK();
 
-	CdromLabel[0] = '\0';
-	CdromId[0] = '\0';
+	memset(CdromLabel, 0, sizeof(CdromLabel));
+	memset(CdromId, 0, sizeof(CdromId));
+	memset(exename, 0, sizeof(exename));
 
 	strncpy(CdromLabel, buf + 52, 32);
 
@@ -294,15 +303,18 @@ int CheckCdrom() {
 			sscanf((char *)buf + 12, "BOOT = cdrom:%256s", exename);
 			if (GetCdromFile(mdir, time, exename) == -1) {
 				char *ptr = strstr(buf + 12, "cdrom:");			// possibly the executable is in some subdir
-				for (i=0; i<32; i++) {
-					if (ptr[i] == ' ') continue;
-					if (ptr[i] == '\\') continue;
-				}
-				if (ptr) {
-					strncpy(exename, ptr, 256);
+				if (ptr != NULL) {
+					ptr += 6;
+					while (*ptr == '\\' || *ptr == '/') ptr++;
+					strncpy(exename, ptr, 255);
+					exename[255] = '\0';
+					ptr = exename;
+					while (*ptr != '\0' && *ptr != '\r' && *ptr != '\n') ptr++;
+					*ptr = '\0';
 					if (GetCdromFile(mdir, time, exename) == -1)
 					 	return -1;		// main executable not found
-				}
+				} else
+					return -1;
 			}
 		}
 	} else if (GetCdromFile(mdir, time, "PSX.EXE;1") != -1) {
@@ -312,28 +324,35 @@ int CheckCdrom() {
 		return -1;		// SYSTEM.CNF and PSX.EXE not found
 
 	if (CdromId[0] == '\0') {
-		i = strlen(exename);
-		if (i >= 2) {
-			if (exename[i - 2] == ';') i-= 2;
-			c = 8; i--;
-			while (i >= 0 && c >= 0) {
-				if (isalnum(exename[i])) CdromId[c--] = exename[i];
-				i--;
-			}
+		len = strlen(exename);
+		c = 0;
+		for (i = 0; i < len; ++i) {
+			if (exename[i] == ';' || c >= sizeof(CdromId) - 1)
+				break;
+			if (isalnum(exename[i])) 
+				CdromId[c++] = exename[i];
 		}
 	}
 
 	if (Config.PsxAuto) { // autodetect system (pal or ntsc)
-		if (strstr(exename, "ES") != NULL)
-			Config.PsxType = 1; // pal
-		else Config.PsxType = 0; // ntsc
+		if((CdromId[2] == 'e') || (CdromId[2] == 'E') ||
+			!strncmp(CdromId, "\0DTLS3035", 10) ||
+			!strncmp(CdromId, "PBPX95001", 10) || // according to redump.org, these PAL
+			!strncmp(CdromId, "PBPX95007", 10) || // discs have a non-standard ID;
+			!strncmp(CdromId, "PBPX95008", 10))   // add more serials if they are discovered.
+			Config.PsxType = PSX_TYPE_PAL; // pal
+		else Config.PsxType = PSX_TYPE_NTSC; // ntsc
 	}
-	psxUpdateVSyncRate();
+
 	if (CdromLabel[0] == ' ') {
 		strncpy(CdromLabel, CdromId, 9);
 	}
-	SysPrintf("CD-ROM Label: %.32s\n", CdromLabel);
-	SysPrintf("CD-ROM ID: %.9s\n", CdromId);
+	SysPrintf(_("CD-ROM Label: %.32s\n"), CdromLabel);
+	SysPrintf(_("CD-ROM ID: %.9s\n"), CdromId);
+	SysPrintf(_("CD-ROM EXE Name: %.255s\n"), exename);
+
+	BuildPPFCache();
+	LoadSBI(NULL);
 
 	return 0;
 }
@@ -363,11 +382,28 @@ static int PSXGetFileType(FILE *f) {
 	return INVALID_EXE;
 }
 
-/* TODO Error handling - return integer for each error case below, defined in an enum. Pass variable on return */
-int Load(char *ExePath) {
+static void LoadLibPS() {
+	char buf[MAXPATHLEN];
+	FILE *f;
+
+	// Load Net Yaroze runtime library (if exists)
+	sprintf(buf, "%s/libps.exe", Config.BiosDir);
+	f = fopen(buf, "rb");
+
+	if (f != NULL) {
+		fseek(f, 0x800, SEEK_SET);
+		fread(psxM + 0x10000, 0x61000, 1, f);
+		fclose(f);
+	}
+}
+
+int Load(const char *ExePath) {
 	FILE *tmpFile;
 	EXE_HEADER tmpHead;
-	int type;
+	FILHDR coffHead;
+	AOUTHDR optHead;
+	SCNHDR section;
+	int type, i;
 	int retval = 0;
 	u8 opcode;
 	u32 section_address, section_size;
@@ -380,12 +416,14 @@ int Load(char *ExePath) {
 		SysPrintf(_("Error opening file: %s.\n"), ExePath);
 		retval = -1;
 	} else {
+		LoadLibPS();
+
 		type = PSXGetFileType(tmpFile);
 		switch (type) {
 			case PSX_EXE:
-				fread(&tmpHead,sizeof(EXE_HEADER),1,tmpFile);
+				fread(&tmpHead, sizeof(EXE_HEADER), 1, tmpFile);
 				fseek(tmpFile, 0x800, SEEK_SET);		
-				fread((void *)PSXM(SWAP32(tmpHead.t_addr)), SWAP32(tmpHead.t_size),1,tmpFile);
+				fread(PSXM(SWAP32(tmpHead.t_addr)), SWAP32(tmpHead.t_size), 1, tmpFile);
 				fclose(tmpFile);
 				psxRegs.pc = SWAP32(tmpHead.pc0);
 				psxRegs.GPR.n.gp = SWAP32(tmpHead.gp0);
@@ -394,6 +432,7 @@ int Load(char *ExePath) {
 					psxRegs.GPR.n.sp = 0x801fff00;
 				retval = 0;
 				break;
+
 			case CPE_EXE:
 				fseek(tmpFile, 6, SEEK_SET); /* Something tells me we should go to 4 and read the "08 00" here... */
 				do {
@@ -423,12 +462,29 @@ int Load(char *ExePath) {
 					}
 				} while (opcode != 0 && retval == 0);
 				break;
+
 			case COFF_EXE:
-				SysPrintf(_("COFF files not supported.\n"));
-				retval = -1;
+				fread(&coffHead, sizeof(coffHead), 1, tmpFile);
+				fread(&optHead, sizeof(optHead), 1, tmpFile);
+
+				psxRegs.pc = SWAP32(optHead.entry);
+				psxRegs.GPR.n.sp = 0x801fff00;
+
+				for (i = 0; i < SWAP16(coffHead.f_nscns); i++) {
+					fseek(tmpFile, sizeof(FILHDR) + SWAP16(coffHead.f_opthdr) + sizeof(section) * i, SEEK_SET);
+					fread(&section, sizeof(section), 1, tmpFile);
+
+					if (section.s_scnptr != 0) {
+						fseek(tmpFile, SWAP32(section.s_scnptr), SEEK_SET);
+						fread(PSXM(SWAP32(section.s_paddr)), SWAP32(section.s_size), 1, tmpFile);
+					} else {
+						memset(PSXM(SWAP32(section.s_paddr)), 0, SWAP32(section.s_size));
+					}
+				}
 				break;
+
 			case INVALID_EXE:
-				SysPrintf(_("This file does not appear to be a valid PSX file.\n"));
+				SysPrintf("%s", _("This file does not appear to be a valid PSX file.\n"));
 				retval = -1;
 				break;
 		}
@@ -444,9 +500,13 @@ int Load(char *ExePath) {
 
 // STATES
 
-const char PcsxHeader[32] = "STv3 PCSX v" PACKAGE_VERSION;
+static const char PcsxrHeader[32] = "STv4 PCSXR v" PACKAGE_VERSION;
 
-int SaveState(char *file) {
+// Savestate Versioning!
+// If you make changes to the savestate version, please increment the value below.
+static const u32 SaveVersion = 0x8b410008;
+
+int SaveState(const char *file) {
 	gzFile f;
 	GPUFreeze_t *gpufP;
 	SPUFreeze_t *spufP;
@@ -456,12 +516,14 @@ int SaveState(char *file) {
 	f = gzopen(file, "wb");
 	if (f == NULL) return -1;
 
-	gzwrite(f, (void*)PcsxHeader, 32);
+	gzwrite(f, (void *)PcsxrHeader, 32);
+	gzwrite(f, (void *)&SaveVersion, sizeof(u32));
+	gzwrite(f, (void *)&Config.HLE, sizeof(boolean));
 
-	pMem = (unsigned char *) malloc(128*96*3);
+	pMem = (unsigned char *)malloc(128 * 96 * 3);
 	if (pMem == NULL) return -1;
 	GPU_getScreenPic(pMem);
-	gzwrite(f, pMem, 128*96*3);
+	gzwrite(f, pMem, 128 * 96 * 3);
 	free(pMem);
 
 	if (Config.HLE)
@@ -470,19 +532,21 @@ int SaveState(char *file) {
 	gzwrite(f, psxM, 0x00200000);
 	gzwrite(f, psxR, 0x00080000);
 	gzwrite(f, psxH, 0x00010000);
-	gzwrite(f, (void*)&psxRegs, sizeof(psxRegs));
+	gzwrite(f, (void *)&psxRegs, sizeof(psxRegs));
 
 	// gpu
-	gpufP = (GPUFreeze_t *) malloc(sizeof(GPUFreeze_t));
+	gpufP = (GPUFreeze_t *)malloc(sizeof(GPUFreeze_t));
 	gpufP->ulFreezeVersion = 1;
 	GPU_freeze(1, gpufP);
 	gzwrite(f, gpufP, sizeof(GPUFreeze_t));
 	free(gpufP);
 
 	// spu
-	spufP = (SPUFreeze_t *) malloc(16);
+	spufP = (SPUFreeze_t *) malloc(16); // only first 3 elements (up to Size)
 	SPU_freeze(2, spufP);
 	Size = spufP->Size; gzwrite(f, &Size, 4);
+	if (Size <= 0)
+		return 1; // error
 	free(spufP);
 	spufP = (SPUFreeze_t *) malloc(Size);
 	SPU_freeze(1, spufP);
@@ -500,41 +564,47 @@ int SaveState(char *file) {
 	return 0;
 }
 
-int LoadState(char *file) {
+int LoadState(const char *file) {
 	gzFile f;
 	GPUFreeze_t *gpufP;
 	SPUFreeze_t *spufP;
 	int Size;
 	char header[32];
+	u32 version;
+	boolean hle;
 
 	f = gzopen(file, "rb");
 	if (f == NULL) return -1;
 
+	gzread(f, header, sizeof(header));
+	gzread(f, &version, sizeof(u32));
+	gzread(f, &hle, sizeof(boolean));
+
+	if (strncmp("STv4 PCSXR", header, 10) != 0 || version != SaveVersion || hle != Config.HLE) {
+		gzclose(f);
+		return -1;
+	}
+
 	psxCpu->Reset();
-
-	gzread(f, header, 32);
-
-	if (strncmp("STv3 PCSX", header, 9)) { gzclose(f); return -1; }
-
-	gzseek(f, 128*96*3, SEEK_CUR);
+	gzseek(f, 128 * 96 * 3, SEEK_CUR);
 
 	gzread(f, psxM, 0x00200000);
 	gzread(f, psxR, 0x00080000);
 	gzread(f, psxH, 0x00010000);
-	gzread(f, (void*)&psxRegs, sizeof(psxRegs));
+	gzread(f, (void *)&psxRegs, sizeof(psxRegs));
 
 	if (Config.HLE)
 		psxBiosFreeze(0);
 
 	// gpu
-	gpufP = (GPUFreeze_t *) malloc (sizeof(GPUFreeze_t));
+	gpufP = (GPUFreeze_t *)malloc(sizeof(GPUFreeze_t));
 	gzread(f, gpufP, sizeof(GPUFreeze_t));
 	GPU_freeze(0, gpufP);
 	free(gpufP);
 
 	// spu
 	gzread(f, &Size, 4);
-	spufP = (SPUFreeze_t *) malloc (Size);
+	spufP = (SPUFreeze_t *)malloc(Size);
 	gzread(f, spufP, Size);
 	SPU_freeze(0, spufP);
 	free(spufP);
@@ -550,20 +620,23 @@ int LoadState(char *file) {
 	return 0;
 }
 
-int CheckState(char *file) {
+int CheckState(const char *file) {
 	gzFile f;
 	char header[32];
+	u32 version;
+	boolean hle;
 
 	f = gzopen(file, "rb");
 	if (f == NULL) return -1;
 
-	psxCpu->Reset();
-
-	gzread(f, header, 32);
+	gzread(f, header, sizeof(header));
+	gzread(f, &version, sizeof(u32));
+	gzread(f, &hle, sizeof(boolean));
 
 	gzclose(f);
 
-	if (strncmp("STv3 PCSX", header, 9)) return -1;
+	if (strncmp("STv4 PCSXR", header, 10) != 0 || version != SaveVersion || hle != Config.HLE)
+		return -1;
 
 	return 0;
 }
@@ -574,17 +647,13 @@ int SendPcsxInfo() {
 	if (NET_recvData == NULL || NET_sendData == NULL)
 		return 0;
 
-//	SysPrintf("SendPcsxInfo\n");
-
 	NET_sendData(&Config.Xa, sizeof(Config.Xa), PSE_NET_BLOCKING);
-	NET_sendData(&Config.Sio, sizeof(Config.Sio), PSE_NET_BLOCKING);
+	NET_sendData(&Config.SioIrq, sizeof(Config.SioIrq), PSE_NET_BLOCKING);
 	NET_sendData(&Config.SpuIrq, sizeof(Config.SpuIrq), PSE_NET_BLOCKING);
 	NET_sendData(&Config.RCntFix, sizeof(Config.RCntFix), PSE_NET_BLOCKING);
 	NET_sendData(&Config.PsxType, sizeof(Config.PsxType), PSE_NET_BLOCKING);
 	NET_sendData(&Config.Cpu, sizeof(Config.Cpu), PSE_NET_BLOCKING);
 
-//	SysPrintf("Send OK\n");
-
 	return 0;
 }
 
@@ -594,14 +663,11 @@ int RecvPcsxInfo() {
 	if (NET_recvData == NULL || NET_sendData == NULL)
 		return 0;
 
-//	SysPrintf("RecvPcsxInfo\n");
-
 	NET_recvData(&Config.Xa, sizeof(Config.Xa), PSE_NET_BLOCKING);
-	NET_recvData(&Config.Sio, sizeof(Config.Sio), PSE_NET_BLOCKING);
+	NET_recvData(&Config.SioIrq, sizeof(Config.SioIrq), PSE_NET_BLOCKING);
 	NET_recvData(&Config.SpuIrq, sizeof(Config.SpuIrq), PSE_NET_BLOCKING);
 	NET_recvData(&Config.RCntFix, sizeof(Config.RCntFix), PSE_NET_BLOCKING);
 	NET_recvData(&Config.PsxType, sizeof(Config.PsxType), PSE_NET_BLOCKING);
-	psxUpdateVSyncRate();
 
 	SysUpdate();
 
@@ -610,8 +676,7 @@ int RecvPcsxInfo() {
 	if (tmp != Config.Cpu) {
 		psxCpu->Shutdown();
 #ifdef PSXREC
-		if (Config.Cpu)	
-			psxCpu = &psxInt;
+		if (Config.Cpu == CPU_INTERPRETER) psxCpu = &psxInt;
 		else psxCpu = &psxRec;
 #else
 		psxCpu = &psxInt;
@@ -622,27 +687,9 @@ int RecvPcsxInfo() {
 		psxCpu->Reset();
 	}
 
-//	SysPrintf("Recv OK\n");
-
 	return 0;
 }
 
-void __Log(char *fmt, ...) {
-	va_list list;
-#ifdef LOG_STDOUT
-	char tmp[1024];
-#endif
-
-	va_start(list, fmt);
-#ifndef LOG_STDOUT
-	vfprintf(emuLog, fmt, list);
-#else
-	vsprintf(tmp, fmt, list);
-	SysPrintf(tmp);
-#endif
-	va_end(list);
-}
-
 // remove the leading and trailing spaces in a string
 void trim(char *str) {
 	int pos = 0;
@@ -663,3 +710,47 @@ void trim(char *str) {
 	while (dest >= str && *dest <= ' ' && *dest > 0)
 		*(dest--) = '\0';
 }
+
+// lookup table for crc calculation
+static unsigned short crctab[256] = {
+	0x0000, 0x1021, 0x2042, 0x3063, 0x4084, 0x50A5, 0x60C6, 0x70E7, 0x8108,
+	0x9129, 0xA14A, 0xB16B, 0xC18C, 0xD1AD, 0xE1CE, 0xF1EF, 0x1231, 0x0210,
+	0x3273, 0x2252, 0x52B5, 0x4294, 0x72F7, 0x62D6, 0x9339, 0x8318, 0xB37B,
+	0xA35A, 0xD3BD, 0xC39C, 0xF3FF, 0xE3DE, 0x2462, 0x3443, 0x0420, 0x1401,
+	0x64E6, 0x74C7, 0x44A4, 0x5485, 0xA56A, 0xB54B, 0x8528, 0x9509, 0xE5EE,
+	0xF5CF, 0xC5AC, 0xD58D, 0x3653, 0x2672, 0x1611, 0x0630, 0x76D7, 0x66F6,
+	0x5695, 0x46B4, 0xB75B, 0xA77A, 0x9719, 0x8738, 0xF7DF, 0xE7FE, 0xD79D,
+	0xC7BC, 0x48C4, 0x58E5, 0x6886, 0x78A7, 0x0840, 0x1861, 0x2802, 0x3823,
+	0xC9CC, 0xD9ED, 0xE98E, 0xF9AF, 0x8948, 0x9969, 0xA90A, 0xB92B, 0x5AF5,
+	0x4AD4, 0x7AB7, 0x6A96, 0x1A71, 0x0A50, 0x3A33, 0x2A12, 0xDBFD, 0xCBDC,
+	0xFBBF, 0xEB9E, 0x9B79, 0x8B58, 0xBB3B, 0xAB1A, 0x6CA6, 0x7C87, 0x4CE4,
+	0x5CC5, 0x2C22, 0x3C03, 0x0C60, 0x1C41, 0xEDAE, 0xFD8F, 0xCDEC, 0xDDCD,
+	0xAD2A, 0xBD0B, 0x8D68, 0x9D49, 0x7E97, 0x6EB6, 0x5ED5, 0x4EF4, 0x3E13,
+	0x2E32, 0x1E51, 0x0E70, 0xFF9F, 0xEFBE, 0xDFDD, 0xCFFC, 0xBF1B, 0xAF3A,
+	0x9F59, 0x8F78, 0x9188, 0x81A9, 0xB1CA, 0xA1EB, 0xD10C, 0xC12D, 0xF14E,
+	0xE16F, 0x1080, 0x00A1, 0x30C2, 0x20E3, 0x5004, 0x4025, 0x7046, 0x6067,
+	0x83B9, 0x9398, 0xA3FB, 0xB3DA, 0xC33D, 0xD31C, 0xE37F, 0xF35E, 0x02B1,
+	0x1290, 0x22F3, 0x32D2, 0x4235, 0x5214, 0x6277, 0x7256, 0xB5EA, 0xA5CB,
+	0x95A8, 0x8589, 0xF56E, 0xE54F, 0xD52C, 0xC50D, 0x34E2, 0x24C3, 0x14A0,
+	0x0481, 0x7466, 0x6447, 0x5424, 0x4405, 0xA7DB, 0xB7FA, 0x8799, 0x97B8,
+	0xE75F, 0xF77E, 0xC71D, 0xD73C, 0x26D3, 0x36F2, 0x0691, 0x16B0, 0x6657,
+	0x7676, 0x4615, 0x5634, 0xD94C, 0xC96D, 0xF90E, 0xE92F, 0x99C8, 0x89E9,
+	0xB98A, 0xA9AB, 0x5844, 0x4865, 0x7806, 0x6827, 0x18C0, 0x08E1, 0x3882,
+	0x28A3, 0xCB7D, 0xDB5C, 0xEB3F, 0xFB1E, 0x8BF9, 0x9BD8, 0xABBB, 0xBB9A,
+	0x4A75, 0x5A54, 0x6A37, 0x7A16, 0x0AF1, 0x1AD0, 0x2AB3, 0x3A92, 0xFD2E,
+	0xED0F, 0xDD6C, 0xCD4D, 0xBDAA, 0xAD8B, 0x9DE8, 0x8DC9, 0x7C26, 0x6C07,
+	0x5C64, 0x4C45, 0x3CA2, 0x2C83, 0x1CE0, 0x0CC1, 0xEF1F, 0xFF3E, 0xCF5D,
+	0xDF7C, 0xAF9B, 0xBFBA, 0x8FD9, 0x9FF8, 0x6E17, 0x7E36, 0x4E55, 0x5E74,
+	0x2E93, 0x3EB2, 0x0ED1, 0x1EF0
+};
+
+u16 calcCrc(u8 *d, int len) {
+	u16 crc = 0;
+	int i;
+
+	for (i = 0; i < len; i++) {
+		crc = crctab[(crc >> 8) ^ d[i]] ^ (crc << 8);
+	}
+
+	return ~crc;
+}
