@@ -13,7 +13,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Steet, Fifth Floor, Boston, MA 02111-1307 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <windows.h>
@@ -25,16 +25,25 @@
 #include "resource.h"
 #include "Win32.h"
 
-#define QueryKeyV(s, name, var) \
-	size = s; \
-	if (RegQueryValueEx(myKey, name, 0, &type, (LPBYTE) var, &size) != 0) { if (err) { RegCloseKey(myKey); return -1; } }
+#define QueryKeyV(name, var) \
+	size = sizeof(DWORD); \
+	if (RegQueryValueEx(myKey, name, 0, &type, (LPBYTE)&tmp, &size) != 0) { if (err) { RegCloseKey(myKey); return -1; } } \
+	var = tmp;
 
-#define SetKeyV(name, var, s, t) \
-	RegSetValueEx(myKey, name, 0, t, (LPBYTE) var, s);
+#define QueryKey(s, name, var) \
+	size = s; \
+	if (RegQueryValueEx(myKey, name, 0, &type, (LPBYTE)var, &size) != 0) { if (err) { RegCloseKey(myKey); return -1; } }
+
+#define SetKeyV(name, var) \
+	tmp = var; \
+	RegSetValueEx(myKey, name, 0, REG_DWORD, (LPBYTE)&tmp, sizeof(DWORD));
+
+#define SetKey(name, var, s, t) \
+	RegSetValueEx(myKey, name, 0, t, (LPBYTE)var, s);
 
 int LoadConfig() {
 	HKEY myKey;
-	DWORD type,size;
+	DWORD type, size, tmp;
 	PcsxConfig *Conf = &Config;
 	int err;
 #ifdef ENABLE_NLS
@@ -44,30 +53,43 @@ int LoadConfig() {
 	if (RegOpenKeyEx(HKEY_CURRENT_USER,cfgfile,0,KEY_ALL_ACCESS,&myKey)!=ERROR_SUCCESS) return -1;
 
 	err = 1;
-	QueryKeyV(256, "Bios", Conf->Bios);
-	QueryKeyV(256, "Gpu",  Conf->Gpu);
-	QueryKeyV(256, "Spu",  Conf->Spu);
-	QueryKeyV(256, "Cdr",  Conf->Cdr);
-	QueryKeyV(256, "Pad1", Conf->Pad1);
-	QueryKeyV(256, "Pad2", Conf->Pad2);
-	QueryKeyV(256, "Mcd1", Conf->Mcd1);
-	QueryKeyV(256, "Mcd2", Conf->Mcd2);
-	QueryKeyV(256, "PluginsDir", Conf->PluginsDir);
-	QueryKeyV(256, "BiosDir",    Conf->BiosDir);
+	QueryKey(256, "Bios", Conf->Bios);
+	QueryKey(256, "Gpu",  Conf->Gpu);
+	QueryKey(256, "Spu",  Conf->Spu);
+	QueryKey(256, "Cdr",  Conf->Cdr);
+	QueryKey(256, "Pad1", Conf->Pad1);
+	QueryKey(256, "Pad2", Conf->Pad2);
+#ifdef ENABLE_SIO1API
+	QueryKey(256, "Sio1", Conf->Sio1);
+#endif
+	QueryKey(256, "Mcd1", Conf->Mcd1);
+	QueryKey(256, "Mcd2", Conf->Mcd2);
+	QueryKey(256, "PluginsDir", Conf->PluginsDir);
+	QueryKey(256, "BiosDir",    Conf->BiosDir);
 	err = 0;
-	QueryKeyV(256, "Net",  Conf->Net);
-	QueryKeyV(256, "Lang", Conf->Lang);
-	QueryKeyV(sizeof(Conf->Xa),      "Xa",      &Conf->Xa);
-	QueryKeyV(sizeof(Conf->Sio),     "Sio",     &Conf->Sio);
-	QueryKeyV(sizeof(Conf->Mdec),    "Mdec",    &Conf->Mdec);
-	QueryKeyV(sizeof(Conf->PsxAuto), "PsxAuto", &Conf->PsxAuto);
-	QueryKeyV(sizeof(Conf->PsxType), "PsxType", &Conf->PsxType);
-	QueryKeyV(sizeof(Conf->Cdda),    "Cdda",    &Conf->Cdda);
-	QueryKeyV(sizeof(Conf->Cpu),     "Cpu",     &Conf->Cpu);
-	QueryKeyV(sizeof(Conf->PsxOut),  "PsxOut",  &Conf->PsxOut);
-	QueryKeyV(sizeof(Conf->SpuIrq),  "SpuIrq",  &Conf->SpuIrq);
-	QueryKeyV(sizeof(Conf->RCntFix), "RCntFix", &Conf->RCntFix);
-	QueryKeyV(sizeof(Conf->VSyncWA), "VSyncWA", &Conf->VSyncWA);
+	QueryKey(256, "Net",  Conf->Net);
+	QueryKey(256, "Lang", Conf->Lang);
+
+	QueryKeyV("Xa",       Conf->Xa);
+	QueryKeyV("SioIrq",   Conf->SioIrq);
+	QueryKeyV("Mdec",     Conf->Mdec);
+	QueryKeyV("PsxAuto",  Conf->PsxAuto);
+	QueryKeyV("Cdda",     Conf->Cdda);
+	QueryKeyV("SlowBoot", Conf->SlowBoot);
+	QueryKeyV("Debug",    Conf->Debug);
+	QueryKeyV("PsxOut",   Conf->PsxOut);
+	QueryKeyV("SpuIrq",   Conf->SpuIrq);
+	QueryKeyV("RCntFix",  Conf->RCntFix);
+	QueryKeyV("VSyncWA",  Conf->VSyncWA);
+	QueryKeyV("Widescreen",  Conf->Widescreen);
+	QueryKeyV("HideCursor",  Conf->HideCursor);
+
+	QueryKeyV("Cpu",      Conf->Cpu);
+	QueryKeyV("PsxType",  Conf->PsxType);
+
+	if (Config.Cpu == CPU_DYNAREC) {
+		Config.Debug = 0; // don't enable debugger if using dynarec core
+	}
 
 	RegCloseKey(myKey);
 
@@ -83,34 +105,41 @@ int LoadConfig() {
 
 void SaveConfig() {
 	HKEY myKey;
-	DWORD myDisp;
+	DWORD myDisp, tmp;
 	PcsxConfig *Conf = &Config;
 
 	RegCreateKeyEx(HKEY_CURRENT_USER, cfgfile, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &myKey, &myDisp);
 
-	SetKeyV("Bios", Conf->Bios, strlen(Conf->Bios), REG_SZ);
-	SetKeyV("Gpu",  Conf->Gpu,  strlen(Conf->Gpu),  REG_SZ);
-	SetKeyV("Spu",  Conf->Spu,  strlen(Conf->Spu),  REG_SZ);
-	SetKeyV("Cdr",  Conf->Cdr,  strlen(Conf->Cdr),  REG_SZ);
-	SetKeyV("Pad1", Conf->Pad1, strlen(Conf->Pad1), REG_SZ);
-	SetKeyV("Pad2", Conf->Pad2, strlen(Conf->Pad2), REG_SZ);
-	SetKeyV("Net",  Conf->Net,  strlen(Conf->Net),  REG_SZ);
-	SetKeyV("Mcd1", Conf->Mcd1, strlen(Conf->Mcd1), REG_SZ);
-	SetKeyV("Mcd2", Conf->Mcd2, strlen(Conf->Mcd2), REG_SZ);
-	SetKeyV("Lang", Conf->Lang, strlen(Conf->Lang), REG_SZ);
-	SetKeyV("PluginsDir", Conf->PluginsDir, strlen(Conf->PluginsDir), REG_SZ);
-	SetKeyV("BiosDir",    Conf->BiosDir,    strlen(Conf->BiosDir), REG_SZ);
-	SetKeyV("Xa",      &Conf->Xa,      sizeof(Conf->Xa),      REG_DWORD);
-	SetKeyV("Sio",     &Conf->Sio,     sizeof(Conf->Sio),     REG_DWORD);
-	SetKeyV("Mdec",    &Conf->Mdec,    sizeof(Conf->Mdec),    REG_DWORD);
-	SetKeyV("PsxAuto", &Conf->PsxAuto, sizeof(Conf->PsxAuto), REG_DWORD);
-	SetKeyV("PsxType", &Conf->PsxType, sizeof(Conf->PsxType), REG_DWORD);
-	SetKeyV("Cdda",    &Conf->Cdda,    sizeof(Conf->Cdda),    REG_DWORD);
-	SetKeyV("Cpu",     &Conf->Cpu,     sizeof(Conf->Cpu),     REG_DWORD);
-	SetKeyV("PsxOut",  &Conf->PsxOut,  sizeof(Conf->PsxOut),  REG_DWORD);
-	SetKeyV("SpuIrq",  &Conf->SpuIrq,  sizeof(Conf->SpuIrq),  REG_DWORD);
-	SetKeyV("RCntFix", &Conf->RCntFix, sizeof(Conf->RCntFix), REG_DWORD);
-	SetKeyV("VSyncWA", &Conf->VSyncWA, sizeof(Conf->VSyncWA), REG_DWORD);
+	SetKey("Bios", Conf->Bios, strlen(Conf->Bios), REG_SZ);
+	SetKey("Gpu",  Conf->Gpu,  strlen(Conf->Gpu),  REG_SZ);
+	SetKey("Spu",  Conf->Spu,  strlen(Conf->Spu),  REG_SZ);
+	SetKey("Cdr",  Conf->Cdr,  strlen(Conf->Cdr),  REG_SZ);
+	SetKey("Pad1", Conf->Pad1, strlen(Conf->Pad1), REG_SZ);
+	SetKey("Pad2", Conf->Pad2, strlen(Conf->Pad2), REG_SZ);
+	SetKey("Sio1", Conf->Sio1, strlen(Conf->Sio1), REG_SZ);
+	SetKey("Net",  Conf->Net,  strlen(Conf->Net),  REG_SZ);
+	SetKey("Mcd1", Conf->Mcd1, strlen(Conf->Mcd1), REG_SZ);
+	SetKey("Mcd2", Conf->Mcd2, strlen(Conf->Mcd2), REG_SZ);
+	SetKey("Lang", Conf->Lang, strlen(Conf->Lang), REG_SZ);
+	SetKey("PluginsDir", Conf->PluginsDir, strlen(Conf->PluginsDir), REG_SZ);
+	SetKey("BiosDir",    Conf->BiosDir,    strlen(Conf->BiosDir), REG_SZ);
+
+	SetKeyV("Xa",      Conf->Xa);
+	SetKeyV("SioIrq",  Conf->SioIrq);
+	SetKeyV("Mdec",    Conf->Mdec);
+	SetKeyV("PsxAuto", Conf->PsxAuto);
+	SetKeyV("Cdda",    Conf->Cdda);
+	SetKeyV("SlowBoot",Conf->SlowBoot);
+	SetKeyV("Debug",   Conf->Debug);
+	SetKeyV("PsxOut",  Conf->PsxOut);
+	SetKeyV("SpuIrq",  Conf->SpuIrq);
+	SetKeyV("RCntFix", Conf->RCntFix);
+	SetKeyV("VSyncWA", Conf->VSyncWA);
+	SetKeyV("Widescreen", Conf->Widescreen);
+	SetKeyV("HideCursor", Conf->HideCursor);
+
+	SetKeyV("Cpu",     Conf->Cpu);
+	SetKeyV("PsxType", Conf->PsxType);
 
 	RegCloseKey(myKey);
 }
@@ -138,6 +167,7 @@ BOOL OnConfigurePluginsDialog(HWND hW) {
 	HWND hWC_CDR=GetDlgItem(hW,IDC_LISTCDR);
 	HWND hWC_PAD1=GetDlgItem(hW,IDC_LISTPAD1);
 	HWND hWC_PAD2=GetDlgItem(hW,IDC_LISTPAD2);
+	HWND hWC_SIO1=GetDlgItem(hW,IDC_LISTSIO1);
 	HWND hWC_BIOS=GetDlgItem(hW,IDC_LISTBIOS);
 	char tmpStr[256];
 	char *lp;
@@ -147,57 +177,61 @@ BOOL OnConfigurePluginsDialog(HWND hW) {
 	strcat(tmpStr, "*.dll");
 	Find = FindFirstFile(tmpStr, &FindData);
 
-	do {
-		if (Find == INVALID_HANDLE_VALUE) break;
-		sprintf(tmpStr,"%s%s", Config.PluginsDir, FindData.cFileName);
-		Lib = LoadLibrary(tmpStr);
-		if (Lib != NULL) {
-			PSE_GetLibType = (PSEgetLibType) GetProcAddress((HMODULE)Lib,"PSEgetLibType");
-			PSE_GetLibName = (PSEgetLibName) GetProcAddress((HMODULE)Lib,"PSEgetLibName");
-			PSE_GetLibVersion = (PSEgetLibVersion) GetProcAddress((HMODULE)Lib,"PSEgetLibVersion");
+	if (Find != INVALID_HANDLE_VALUE) {
+		do {
+			sprintf(tmpStr,"%s%s", Config.PluginsDir, FindData.cFileName);
+			Lib = LoadLibrary(tmpStr);
+			if (Lib != NULL) {
+				PSE_GetLibType = (PSEgetLibType) GetProcAddress((HMODULE)Lib,"PSEgetLibType");
+				PSE_GetLibName = (PSEgetLibName) GetProcAddress((HMODULE)Lib,"PSEgetLibName");
+				PSE_GetLibVersion = (PSEgetLibVersion) GetProcAddress((HMODULE)Lib,"PSEgetLibVersion");
 
-			if (PSE_GetLibType != NULL && PSE_GetLibName != NULL && PSE_GetLibVersion != NULL) {
-				unsigned long version = PSE_GetLibVersion();
-				long type;
+				if (PSE_GetLibType != NULL && PSE_GetLibName != NULL && PSE_GetLibVersion != NULL) {
+					unsigned long version = PSE_GetLibVersion();
+					long type;
 
-				sprintf(tmpStr, "%s %d.%d", PSE_GetLibName(), (int)(version>>8)&0xff, (int)version&0xff);
-				type = PSE_GetLibType();
-				if (type & PSE_LT_CDR) {
-					ComboAddPlugin(hWC_CDR, Config.Cdr);
-				}
+					sprintf(tmpStr, "%s %d.%d", PSE_GetLibName(), (int)(version>>8)&0xff, (int)version&0xff);
+					type = PSE_GetLibType();
+					if (type & PSE_LT_CDR) {
+						ComboAddPlugin(hWC_CDR, Config.Cdr);
+					}
 
-				if (type & PSE_LT_SPU) {
-					ComboAddPlugin(hWC_SPU, Config.Spu);
-				}
+					if (type & PSE_LT_SPU) {
+						ComboAddPlugin(hWC_SPU, Config.Spu);
+					}
 
-				if (type & PSE_LT_GPU) {
-					ComboAddPlugin(hWC_GPU, Config.Gpu);
-				}
+					if (type & PSE_LT_GPU) {
+						ComboAddPlugin(hWC_GPU, Config.Gpu);
+					}
 
-				if (type & PSE_LT_PAD) {
-					PADquery query;
+					if (type & PSE_LT_PAD) {
+						PADquery query;
 
-					query = (PADquery)GetProcAddress((HMODULE)Lib, "PADquery");
-					if (query != NULL) {
-						if (query() & 0x1)
+						query = (PADquery)GetProcAddress((HMODULE)Lib, "PADquery");
+						if (query != NULL) {
+							if (query() & 0x1)
+								ComboAddPlugin(hWC_PAD1, Config.Pad1);
+							if (query() & 0x2)
+								ComboAddPlugin(hWC_PAD2, Config.Pad2);
+						} else { // just a guess
 							ComboAddPlugin(hWC_PAD1, Config.Pad1);
-						if (query() & 0x2)
-							ComboAddPlugin(hWC_PAD2, Config.Pad2);
-					} else { // just a guess
-						ComboAddPlugin(hWC_PAD1, Config.Pad1);
+						}
+					}
+
+					if (type & PSE_LT_SIO1) {
+						ComboAddPlugin(hWC_SIO1, Config.Sio1);
 					}
 				}
 			}
-		}
-	} while (FindNextFile(Find,&FindData));
-
-	if (Find != INVALID_HANDLE_VALUE) FindClose(Find);
+		} while (FindNextFile(Find,&FindData));
+		FindClose(Find);
+	}
 
 // BIOS
 
 	lp = (char *)malloc(strlen("HLE") + 1);
 	sprintf(lp, "HLE");
-	i = ComboBox_AddString(hWC_BIOS, _("Internal HLE Bios"));
+	i = ComboBox_AddString(hWC_BIOS, _("Simulate Psx Bios"));
 	ComboBox_SetItemData(hWC_BIOS, i, lp);
 	if (stricmp(Config.Bios, lp)==0)
 		ComboBox_SetCurSel(hWC_BIOS, i);
@@ -206,23 +240,23 @@ BOOL OnConfigurePluginsDialog(HWND hW) {
 	strcat(tmpStr, "*");
 	Find=FindFirstFile(tmpStr, &FindData);
 
-	do {
-		if (Find==INVALID_HANDLE_VALUE) break;
-		if (!strcmp(FindData.cFileName, ".")) continue;
-		if (!strcmp(FindData.cFileName, "..")) continue;
-		if (FindData.nFileSizeLow != 1024 * 512) continue;
-		lp = (char *)malloc(strlen(FindData.cFileName)+8);
-		sprintf(lp, "%s", (char *)FindData.cFileName);
-		i = ComboBox_AddString(hWC_BIOS, FindData.cFileName);
-		ComboBox_SetItemData(hWC_BIOS, i, lp);
-		if (Config.Bios[0]=='\0') {
-			ComboBox_SetCurSel(hWC_BIOS, i);
-			strcpy(Config.Bios, FindData.cFileName);
-		} else if (stricmp(Config.Bios, FindData.cFileName)==0)
-			ComboBox_SetCurSel(hWC_BIOS, i);
-	} while (FindNextFile(Find,&FindData));
-
-	if (Find!=INVALID_HANDLE_VALUE) FindClose(Find);
+	if (Find != INVALID_HANDLE_VALUE) {
+		do {
+			if (!strcmp(FindData.cFileName, ".")) continue;
+			if (!strcmp(FindData.cFileName, "..")) continue;
+			if (FindData.nFileSizeLow != 1024 * 512) continue;
+			lp = (char *)malloc(strlen(FindData.cFileName)+8);
+			sprintf(lp, "%s", (char *)FindData.cFileName);
+			i = ComboBox_AddString(hWC_BIOS, FindData.cFileName);
+			ComboBox_SetItemData(hWC_BIOS, i, lp);
+			if (Config.Bios[0]=='\0') {
+				ComboBox_SetCurSel(hWC_BIOS, i);
+				strcpy(Config.Bios, FindData.cFileName);
+			} else if (stricmp(Config.Bios, FindData.cFileName)==0)
+				ComboBox_SetCurSel(hWC_BIOS, i);
+		} while (FindNextFile(Find,&FindData));
+		FindClose(Find);
+	}
 
 	if (ComboBox_GetCurSel(hWC_CDR ) == -1)
 		ComboBox_SetCurSel(hWC_CDR,  0);
@@ -234,6 +268,8 @@ BOOL OnConfigurePluginsDialog(HWND hW) {
 		ComboBox_SetCurSel(hWC_PAD1, 0);
 	if (ComboBox_GetCurSel(hWC_PAD2) == -1)
 		ComboBox_SetCurSel(hWC_PAD2, 0);
+	if (ComboBox_GetCurSel(hWC_SIO1 ) == -1)
+		ComboBox_SetCurSel(hWC_SIO1,  0);
 	if (ComboBox_GetCurSel(hWC_BIOS) == -1)
 		ComboBox_SetCurSel(hWC_BIOS, 0);
 
@@ -257,6 +293,7 @@ void CleanUpCombos(HWND hW) {
 	CleanCombo(IDC_LISTCDR);
 	CleanCombo(IDC_LISTPAD1);
 	CleanCombo(IDC_LISTPAD2);
+	CleanCombo(IDC_LISTSIO1);
 	CleanCombo(IDC_LISTBIOS);
 }
 
@@ -279,10 +316,17 @@ void OnOK(HWND hW) {
 	char *cdrDLL=GetSelDLL(hW,IDC_LISTCDR);
 	char *pad1DLL=GetSelDLL(hW,IDC_LISTPAD1);
 	char *pad2DLL=GetSelDLL(hW,IDC_LISTPAD2);
+#ifdef ENABLE_SIO1API
+	char *sio1DLL=GetSelDLL(hW,IDC_LISTSIO1);
+#endif
 	char *biosFILE=GetSelDLL(hW,IDC_LISTBIOS);
 
-    if (gpuDLL == NULL || spuDLL == NULL || cdrDLL == NULL || pad1DLL == NULL ||
-		pad2DLL == NULL || biosFILE == NULL) {
+    if (gpuDLL == NULL || spuDLL == NULL || /*cdrDLL == NULL ||*/ pad1DLL == NULL ||
+		pad2DLL == NULL || biosFILE == NULL
+#ifdef ENABLE_SIO1API
+		|| sio1DLL == NULL
+#endif
+		) {
 		MessageBox(hW, _("Configuration not OK!"), _("Error"), MB_OK | MB_ICONERROR);
 		return;
 	}
@@ -290,9 +334,15 @@ void OnOK(HWND hW) {
 	strcpy(Config.Bios, biosFILE);
 	strcpy(Config.Gpu,  gpuDLL);
 	strcpy(Config.Spu,  spuDLL);
-	strcpy(Config.Cdr,  cdrDLL);
+	if (cdrDLL != NULL)
+		strcpy(Config.Cdr,  cdrDLL);
+	else
+		Config.Cdr[0] = '\0';
 	strcpy(Config.Pad1, pad1DLL);
 	strcpy(Config.Pad2, pad2DLL);
+#ifdef ENABLE_SIO1API
+	strcpy(Config.Sio1, sio1DLL);
+#endif
 
 	SaveConfig();
 
@@ -343,6 +393,11 @@ void ConfigurePAD2(HWND hW) {
 	ConfPlugin(PADconfigure, IDC_LISTPAD2, "PADconfigure");
 }
 
+void ConfigureSIO1(HWND hW) {
+#ifdef ENABLE_SIO1API
+	ConfPlugin(SIO1configure, IDC_LISTSIO1, "SIO1configure");
+#endif
+}
 
 void AboutGPU(HWND hW) {
 	ConfPlugin(GPUabout, IDC_LISTGPU, "GPUabout");
@@ -368,6 +423,12 @@ void AboutPAD2(HWND hW) {
 	ConfPlugin(PADabout, IDC_LISTPAD2, "PADabout");
 }
 
+void AboutSIO1(HWND hW) {
+#ifdef ENABLE_SIO1API
+	ConfPlugin(SIO1about, IDC_LISTSIO1, "SIO1about");
+#endif
+}
+
 
 #define TestPlugin(src, confs, name) \
 	void *drv; \
@@ -413,6 +474,13 @@ void TestPAD2(HWND hW) {
 	TestPlugin(PADtest, IDC_LISTPAD2, "PADtest");
 }
 
+void TestSIO1(HWND hW) {
+#ifdef ENABLE_SIO1API
+	TestPlugin(SIO1test, IDC_LISTSIO1, "SIO1test");
+#endif
+}
+
+
 #include <shlobj.h>
 
 int SelectPath(HWND hW, char *Title, char *Path) {
@@ -468,6 +536,7 @@ BOOL CALLBACK ConfigurePluginsDlgProc(HWND hW, UINT uMsg, WPARAM wParam, LPARAM
 			Static_SetText(GetDlgItem(hW, IDC_SECONDCONTROLLER), _("Second Controller"));
 			Static_SetText(GetDlgItem(hW, IDC_SOUND), _("Sound"));
 			Static_SetText(GetDlgItem(hW, IDC_CDROM), _("Cdrom"));
+			Static_SetText(GetDlgItem(hW, IDC_LINKCABLE), _("Link cable"));
 			Static_SetText(GetDlgItem(hW, IDC_BIOS), _("Bios"));
 			Button_SetText(GetDlgItem(hW, IDC_BIOSDIR), _("Set Bios Directory"));
 			Button_SetText(GetDlgItem(hW, IDC_PLUGINSDIR), _("Set Plugins Directory"));
@@ -486,6 +555,9 @@ BOOL CALLBACK ConfigurePluginsDlgProc(HWND hW, UINT uMsg, WPARAM wParam, LPARAM
 			Button_SetText(GetDlgItem(hW, IDC_CONFIGPAD2), _("Configure..."));
 			Button_SetText(GetDlgItem(hW, IDC_TESTPAD2), _("Test..."));
 			Button_SetText(GetDlgItem(hW, IDC_ABOUTPAD2), _("About..."));
+			Button_SetText(GetDlgItem(hW, IDC_CONFIGSIO1), _("Configure..."));
+			Button_SetText(GetDlgItem(hW, IDC_TESTSIO1), _("Test..."));
+			Button_SetText(GetDlgItem(hW, IDC_ABOUTSIO1), _("About..."));
 
 			return OnConfigurePluginsDialog(hW);
 
@@ -496,18 +568,21 @@ BOOL CALLBACK ConfigurePluginsDlgProc(HWND hW, UINT uMsg, WPARAM wParam, LPARAM
        			case IDC_CONFIGCDR:  ConfigureCDR(hW); return TRUE;
        			case IDC_CONFIGPAD1: ConfigurePAD1(hW); return TRUE;
        			case IDC_CONFIGPAD2: ConfigurePAD2(hW); return TRUE;
+				case IDC_CONFIGSIO1: ConfigureSIO1(hW); return TRUE;
 
 				case IDC_TESTGPU:    TestGPU(hW);   return TRUE;
 				case IDC_TESTSPU:    TestSPU(hW);   return TRUE;
 				case IDC_TESTCDR:    TestCDR(hW);   return TRUE;
 				case IDC_TESTPAD1:   TestPAD1(hW);  return TRUE;
 				case IDC_TESTPAD2:   TestPAD2(hW);  return TRUE;
+				case IDC_TESTSIO1:   TestSIO1(hW);  return TRUE;
 
 				case IDC_ABOUTGPU:   AboutGPU(hW);  return TRUE;
 				case IDC_ABOUTSPU:   AboutSPU(hW);  return TRUE;
                 case IDC_ABOUTCDR:   AboutCDR(hW);  return TRUE;
     	        case IDC_ABOUTPAD1:  AboutPAD1(hW); return TRUE;
     		    case IDC_ABOUTPAD2:  AboutPAD2(hW); return TRUE;
+				case IDC_ABOUTSIO1:  AboutSIO1(hW); return TRUE;
 
 				case IDC_PLUGINSDIR: SetPluginsDir(hW); return TRUE;
 				case IDC_BIOSDIR:	 SetBiosDir(hW);	return TRUE;
@@ -558,30 +633,30 @@ BOOL OnConfigureNetPlayDialog(HWND hW) {
 	ComboBox_SetItemData(hWC_NET, i, lp);
 	ComboBox_SetCurSel(hWC_NET,  0);
 
-	do {
-		if (Find==INVALID_HANDLE_VALUE) break;
-		sprintf(tmpStr,"%s%s", Config.PluginsDir, FindData.cFileName);
-		Lib = LoadLibrary(tmpStr);
-		if (Lib!=NULL) {
-			PSE_GetLibType = (PSEgetLibType) GetProcAddress((HMODULE)Lib,"PSEgetLibType");
-			PSE_GetLibName = (PSEgetLibName) GetProcAddress((HMODULE)Lib,"PSEgetLibName");
-			PSE_GetLibVersion = (PSEgetLibVersion) GetProcAddress((HMODULE)Lib,"PSEgetLibVersion");
+	if (Find != INVALID_HANDLE_VALUE) {
+		do {
+			sprintf(tmpStr,"%s%s", Config.PluginsDir, FindData.cFileName);
+			Lib = LoadLibrary(tmpStr);
+			if (Lib!=NULL) {
+				PSE_GetLibType = (PSEgetLibType) GetProcAddress((HMODULE)Lib,"PSEgetLibType");
+				PSE_GetLibName = (PSEgetLibName) GetProcAddress((HMODULE)Lib,"PSEgetLibName");
+				PSE_GetLibVersion = (PSEgetLibVersion) GetProcAddress((HMODULE)Lib,"PSEgetLibVersion");
 
-			if (PSE_GetLibType != NULL && PSE_GetLibName != NULL && PSE_GetLibVersion != NULL) {
-				unsigned long version = PSE_GetLibVersion();
-				long type;
+				if (PSE_GetLibType != NULL && PSE_GetLibName != NULL && PSE_GetLibVersion != NULL) {
+					unsigned long version = PSE_GetLibVersion();
+					long type;
 
-				sprintf(tmpStr, "%s %d.%d", PSE_GetLibName(), (int)(version>>8)&0xff, (int)version&0xff);
-				type = PSE_GetLibType();
-				if (type & PSE_LT_NET  && ((version >> 16) == 2)) {
-					ComboAddPlugin(hWC_NET, Config.Net);
+					sprintf(tmpStr, "%s %d.%d", PSE_GetLibName(), (int)(version>>8)&0xff, (int)version&0xff);
+					type = PSE_GetLibType();
+					if (type & PSE_LT_NET  && ((version >> 16) == 2)) {
+						ComboAddPlugin(hWC_NET, Config.Net);
+					}
 				}
 			}
-		}
-	} while (FindNextFile(Find,&FindData));
-
-	if (Find!=INVALID_HANDLE_VALUE) FindClose(Find);
-
+		} while (FindNextFile(Find,&FindData));
+		FindClose(Find);
+	}
+	
 	return TRUE;
 }
 
