@@ -13,7 +13,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Steet, Fifth Floor, Boston, MA 02111-1307 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <windows.h>
@@ -50,7 +50,8 @@ int StatesC = 0;
 int CancelQuit = 0;
 char cfgfile[256];
 int Running = 0;
-char PcsxDir[256];
+boolean UseGui = TRUE;
+char PcsxrDir[256];
 
 static HDC          hDC;
 static HDC          hdcmem;
@@ -80,13 +81,14 @@ LangDef sLangs[] = {
 	{ "en",		N_("English"),				0x0409 },
 	{ "es",		N_("Spanish"),				0x040a },
 	{ "fr_FR",	N_("French"),				0x040c },
-	{ "it_IT",	N_("Italian"),				0x0410 },
+	{ "it",		N_("Italian"),				0x0410 },
 	{ "pt",		N_("Portuguese"),			0x0816 },
+	{ "pt_BR",	N_("Portuguese (Brazilian)"),		0x0416 },
 	{ "ro",		N_("Romanian"),				0x0418 },
-	{ "ru",		N_("Russian"),				0x0419 },
+	{ "ru_RU",	N_("Russian"),				0x0419 },
 	{ "zh_CN",	N_("Simplified Chinese"),	0x0804 },
 	{ "zh_TW",	N_("Traditional Chinese"),	0x0404 },
-	{ "jp",		N_("Japanese"),				0x0411 },
+	{ "ja",		N_("Japanese"),				0x0411 },
 	{ "ko",		N_("Korean"),				0x0412 },
 	{ "", "", 0xFFFF },
 };
@@ -129,7 +131,11 @@ void strcatz(char *dst, char *src) {
 }
 
 int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow) {
-	strcpy(cfgfile, "Software\\Pcsx");
+	char *arg = NULL;
+	char cdfile[MAXPATHLEN] = "", buf[4096];
+	int loadstatenum = -1;
+
+	strcpy(cfgfile, "Software\\Pcsxr");
 
 	gApp.hInstance = hInstance;
 
@@ -140,7 +146,7 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine
 
 	Running = 0;
 
-	GetCurrentDirectory(256, PcsxDir);
+	GetCurrentDirectory(256, PcsxrDir);
 
 	memset(&Config, 0, sizeof(PcsxConfig));
 	strcpy(Config.Net, "Disabled");
@@ -170,6 +176,8 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine
 		}
 	}
 
+	strcpy(Config.PatchesDir, "Patches\\");
+
 #ifdef ENABLE_NLS
 	if (Config.Lang[0] == 0) {
 		SetDefaultLang();
@@ -178,10 +186,55 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine
 	}
 #endif
 
+	// Parse command-line
+	strncpy(buf, lpCmdLine, 4096);
+
+	for (arg = strtok(buf, " "); arg != NULL; arg = strtok(NULL, " ")) {
+		if (strcmp(arg, "-nogui") == 0) {
+			UseGui = FALSE;
+		} else if (strcmp(arg, "-runcd") == 0) {
+			cdfile[0] = '\0';
+		} else if (strcmp(arg, "-cdfile") == 0) {
+			arg = strtok(NULL, " ");
+			if (arg != NULL) {
+				if (arg[0] == '"') {
+					strncpy(buf, lpCmdLine + (arg - buf), 4096);
+					arg = strtok(buf, "\"");
+					if (arg != NULL) strcpy(cdfile, arg);
+				} else {
+					strcpy(cdfile, arg);
+				}
+				UseGui = FALSE;
+			}
+		} else if (strcmp(arg, "-psxout") == 0) {
+			Config.PsxOut = TRUE;
+		} else if (strcmp(arg, "-slowboot") == 0) {
+			Config.SlowBoot = TRUE;
+		} else if (strcmp(arg, "-help") == 0) {
+			MessageBox(gApp.hWnd, _(
+				"Usage: pcsxr [options]\n"
+				"\toptions:\n"
+				"\t-nogui\t\tDon't open the GUI\n"
+				"\t-psxout\t\tEnable PSX output\n"
+				"\t-slowboot\tEnable BIOS logo\n"
+				"\t-runcd\t\tRuns CD-ROM (requires -nogui)\n"
+				"\t-cdfile FILE\tRuns a CD image file (requires -nogui)\n"
+				"\t-help\t\tDisplay this message"),
+				"PCSXR", 0);
+
+			return 0;
+		}
+	}
+
 	if (SysInit() == -1) return 1;
 
 	CreateMainWindow(nCmdShow);
 
+	if (!UseGui) {
+		SetIsoFile(cdfile);
+		PostMessage(gApp.hWnd, WM_COMMAND, ID_FILE_RUN_NOGUI, 0);
+	}
+
 	RunGui();
 
 	return 0;
@@ -191,10 +244,15 @@ void RunGui() {
 	MSG msg;
 
 	for (;;) {
-		if(PeekMessage(&msg, NULL, 0U, 0U, PM_REMOVE)) {
+		if (PeekMessage(&msg, NULL, 0U, 0U, PM_REMOVE)) {
 			TranslateMessage(&msg);
 			DispatchMessage(&msg);
 		}
+		else
+		{
+			// Avoid 100% cpu usage.
+			Sleep(10);
+		}
 	}
 }
 
@@ -202,31 +260,26 @@ void RestoreWindow() {
 	AccBreak = 1;
 	DestroyWindow(gApp.hWnd);
 	CreateMainWindow(SW_SHOWNORMAL);
-	ShowCursor(TRUE);
-	SetCursor(LoadCursor(gApp.hInstance, IDC_ARROW));
-	ShowCursor(TRUE);
-}
+	
+	if(Config.HideCursor)
+		ShowCursor(TRUE);
+	
+	//SetCursor(LoadCursor(gApp.hInstance, IDC_ARROW));
+	//ShowCursor(TRUE);
 
-int Slots[5] = { -1, -1, -1, -1, -1 };
+	if (!UseGui) PostMessage(gApp.hWnd, WM_COMMAND, ID_FILE_EXIT, 0);
+}
 
 void ResetMenuSlots() {
-	int i;
-
-	for (i = 0; i < 5; i++) {
-		if (Slots[i] == -1)
-			EnableMenuItem(GetSubMenu(gApp.hMenu, 0), ID_FILE_STATES_LOAD_SLOT1+i, MF_GRAYED);
-		else 
-			EnableMenuItem(GetSubMenu(gApp.hMenu, 0), ID_FILE_STATES_LOAD_SLOT1+i, MF_ENABLED);
-	}
-}
-
-void UpdateMenuSlots() {
 	char str[256];
 	int i;
 
-	for (i = 0; i < 5; i++) {
+	for (i = 0; i < 9; i++) {
 		GetStateFilename(str, i);
-		Slots[i] = CheckState(str);
+		if (CheckState(str) == -1)
+			EnableMenuItem(gApp.hMenu, ID_FILE_STATES_LOAD_SLOT1+i, MF_GRAYED);
+		else 
+			EnableMenuItem(gApp.hMenu, ID_FILE_STATES_LOAD_SLOT1+i, MF_ENABLED);
 	}
 }
 
@@ -253,8 +306,8 @@ void States_Load(int num) {
 
 	ret = LoadState(Text);
 	if (ret == 0)
-		 sprintf(Text, _("*PCSX*: Loaded State %d"), num+1);
-	else sprintf(Text, _("*PCSX*: Error Loading State %d"), num+1);
+		 sprintf(Text, _("*PCSXR*: Loaded State %d"), num+1);
+	else sprintf(Text, _("*PCSXR*: Error Loading State %d"), num+1);
 	GPU_displayText(Text);
 
 	Running = 1;
@@ -275,8 +328,8 @@ void States_Save(int num) {
 	GPU_freeze(2, (GPUFreeze_t *)&num);
 	ret = SaveState(Text);
 	if (ret == 0)
-		 sprintf(Text, _("*PCSX*: Saved State %d"), num+1);
-	else sprintf(Text, _("*PCSX*: Error Saving State %d"), num+1);
+		 sprintf(Text, _("*PCSXR*: Saved State %d"), num+1);
+	else sprintf(Text, _("*PCSXR*: Error Saving State %d"), num+1);
 	GPU_displayText(Text);
 
 	Running = 1;
@@ -294,7 +347,7 @@ void OnStates_LoadOther() {
 	memset(&szFileTitle, 0, sizeof(szFileTitle));
 	memset(&szFilter,    0, sizeof(szFilter));
 
-	strcpy(szFilter, _("PCSX State Format"));
+	strcpy(szFilter, _("PCSXR State Format"));
 	strcatz(szFilter, "*.*");
 
 	ofn.lStructSize			= sizeof(OPENFILENAME);
@@ -321,8 +374,8 @@ void OnStates_LoadOther() {
 
 		ret = LoadState(szFileName);
 		if (ret == 0)
-			 sprintf(Text, _("*PCSX*: Loaded State %s"), szFileName);
-		else sprintf(Text, _("*PCSX*: Error Loading State %s"), szFileName);
+			 sprintf(Text, _("*PCSXR*: Loaded State %s"), szFileName);
+		else sprintf(Text, _("*PCSXR*: Error Loading State %s"), szFileName);
 		GPU_displayText(Text);
 
 		Running = 1;
@@ -330,12 +383,6 @@ void OnStates_LoadOther() {
 	}
 } 
 
-void OnStates_Save1() { States_Save(0); } 
-void OnStates_Save2() { States_Save(1); } 
-void OnStates_Save3() { States_Save(2); } 
-void OnStates_Save4() { States_Save(3); } 
-void OnStates_Save5() { States_Save(4); } 
-
 void OnStates_SaveOther() {
 	OPENFILENAME ofn;
 	char szFileName[MAXPATHLEN];
@@ -346,7 +393,7 @@ void OnStates_SaveOther() {
 	memset(&szFileTitle, 0, sizeof(szFileTitle));
 	memset(&szFilter,    0, sizeof(szFilter));
 
-	strcpy(szFilter, _("PCSX State Format"));
+	strcpy(szFilter, _("PCSXR State Format"));
 	strcatz(szFilter, "*.*");
 
 	ofn.lStructSize			= sizeof(OPENFILENAME);
@@ -373,8 +420,8 @@ void OnStates_SaveOther() {
 
 		ret = SaveState(szFileName);
 		if (ret == 0)
-			 sprintf(Text, _("*PCSX*: Saved State %s"), szFileName);
-		else sprintf(Text, _("*PCSX*: Error Saving State %s"), szFileName);
+			 sprintf(Text, _("*PCSXR*: Saved State %s"), szFileName);
+		else sprintf(Text, _("*PCSXR*: Error Saving State %s"), szFileName);
 		GPU_displayText(Text);
 
 		Running = 1;
@@ -411,7 +458,7 @@ LRESULT WINAPI MainWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {
 					return TRUE;
 
 				case ID_FILE_RUN_CD:
-					cdrfilename[0] = '\0';
+					SetIsoFile(NULL);
 					SetMenu(hWnd, NULL);
 					LoadPlugins();
 					if (OpenPlugins(hWnd) == -1) {
@@ -419,20 +466,24 @@ LRESULT WINAPI MainWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {
 						RestoreWindow();
 						return TRUE;
 					}
-					SysReset();
 					if (CheckCdrom() == -1) {
 						ClosePlugins();
 						RestoreWindow();
 						SysMessage(_("The CD does not appear to be a valid Playstation CD"));
 						return TRUE;
 					}
+					
+					// Auto-detect: region first, then rcnt reset
+					SysReset();
+
 					if (LoadCdrom() == -1) {
 						ClosePlugins();
 						RestoreWindow();
 						SysMessage(_("Could not load CD-ROM!"));
 						return TRUE;
 					}
-					ShowCursor(FALSE);
+					if(Config.HideCursor)
+						ShowCursor(FALSE);
 					Running = 1;
 					psxCpu->Execute();
 					return TRUE;
@@ -442,7 +493,7 @@ LRESULT WINAPI MainWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {
 						SysMessage(_("Running BIOS is not supported with Internal HLE Bios."));
 						return TRUE;
 					}
-					cdrfilename[0] = '\0';
+					SetIsoFile(NULL);
 					SetMenu(hWnd, NULL);
 					LoadPlugins();
 					if (OpenPlugins(hWnd) == -1) {
@@ -450,7 +501,8 @@ LRESULT WINAPI MainWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {
 						RestoreWindow();
 						return TRUE;
 					}
-					ShowCursor(FALSE);
+					if(Config.HideCursor)
+						ShowCursor(FALSE);
 					SysReset();
 					CdromId[0] = '\0';
 					CdromLabel[0] = '\0';
@@ -460,7 +512,7 @@ LRESULT WINAPI MainWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {
 
 				case ID_FILE_RUN_ISO:
 					if (!Open_Iso_Proc(File)) return TRUE;
-					strcpy(cdrfilename, File);
+					SetIsoFile(File);
 					SetMenu(hWnd, NULL);
 					LoadPlugins();
 					if (OpenPlugins(hWnd) == -1) {
@@ -468,27 +520,31 @@ LRESULT WINAPI MainWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {
 						RestoreWindow();
 						return TRUE;
 					}
-					SysReset();
 					if (CheckCdrom() == -1) {
 						ClosePlugins();
 						RestoreWindow();
 						SysMessage(_("The CD does not appear to be a valid Playstation CD"));
 						return TRUE;
 					}
+
+					// Auto-detect: region first, then rcnt reset
+					SysReset();
+					
 					if (LoadCdrom() == -1) {
 						ClosePlugins();
 						RestoreWindow();
 						SysMessage(_("Could not load CD-ROM!"));
 						return TRUE;
 					}
-					ShowCursor(FALSE);
+					if(Config.HideCursor)
+						ShowCursor(FALSE);
 					Running = 1;
 					psxCpu->Execute();
 					return TRUE;
 
 				case ID_FILE_RUN_EXE:
 					if (!Open_File_Proc(File)) return TRUE;
-					cdrfilename[0] = '\0';
+					SetIsoFile(NULL);
 					SetMenu(hWnd, NULL);
 					LoadPlugins();
 					if (OpenPlugins(hWnd) == -1) {
@@ -496,8 +552,11 @@ LRESULT WINAPI MainWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {
 						RestoreWindow();
 						return TRUE;
 					}
-					SysReset();
 					CheckCdrom();
+
+					// Auto-detect: region first, then rcnt reset
+					SysReset();
+					
 					Load(File);
 					Running = 1;
 					psxCpu->Execute();
@@ -508,6 +567,10 @@ LRESULT WINAPI MainWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {
 				case ID_FILE_STATES_LOAD_SLOT3: States_Load(2); return TRUE;
 				case ID_FILE_STATES_LOAD_SLOT4: States_Load(3); return TRUE;
 				case ID_FILE_STATES_LOAD_SLOT5: States_Load(4); return TRUE;
+				case ID_FILE_STATES_LOAD_SLOT6: States_Load(5); return TRUE;
+				case ID_FILE_STATES_LOAD_SLOT7: States_Load(6); return TRUE;
+				case ID_FILE_STATES_LOAD_SLOT8: States_Load(7); return TRUE;
+				case ID_FILE_STATES_LOAD_SLOT9: States_Load(8); return TRUE;
 				case ID_FILE_STATES_LOAD_OTHER: OnStates_LoadOther(); return TRUE;
 
 				case ID_FILE_STATES_SAVE_SLOT1: States_Save(0); return TRUE;
@@ -515,12 +578,47 @@ LRESULT WINAPI MainWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {
 				case ID_FILE_STATES_SAVE_SLOT3: States_Save(2); return TRUE;
 				case ID_FILE_STATES_SAVE_SLOT4: States_Save(3); return TRUE;
 				case ID_FILE_STATES_SAVE_SLOT5: States_Save(4); return TRUE;
+				case ID_FILE_STATES_SAVE_SLOT6: States_Save(5); return TRUE;
+				case ID_FILE_STATES_SAVE_SLOT7: States_Save(6); return TRUE;
+				case ID_FILE_STATES_SAVE_SLOT8: States_Save(7); return TRUE;
+				case ID_FILE_STATES_SAVE_SLOT9: States_Save(8); return TRUE;
 				case ID_FILE_STATES_SAVE_OTHER: OnStates_SaveOther(); return TRUE;
 
+				case ID_FILE_RUN_NOGUI:
+					SetMenu(hWnd, NULL);
+					LoadPlugins();
+					if (OpenPlugins(hWnd) == -1) {
+						ClosePlugins();
+						RestoreWindow();
+						return TRUE;
+					}
+					if (CheckCdrom() == -1) {
+						fprintf(stderr, _("The CD does not appear to be a valid Playstation CD"));
+						ClosePlugins();
+						RestoreWindow();
+						return TRUE;
+					}
+
+					// Auto-detect: region first, then rcnt reset
+					SysReset();
+					
+					if (LoadCdrom() == -1) {
+						fprintf(stderr, _("Could not load CD-ROM!"));
+						ClosePlugins();
+						RestoreWindow();
+						return TRUE;
+					}
+					if(Config.HideCursor)
+						ShowCursor(FALSE);
+					Running = 1;
+					psxCpu->Execute();
+					return TRUE;
+
 				case ID_EMULATOR_RUN:
 					SetMenu(hWnd, NULL);
 					OpenPlugins(hWnd);
-					ShowCursor(FALSE);
+					if(Config.HideCursor)
+						ShowCursor(FALSE);
 					Running = 1;
 					CheatSearchBackupMemory();
 					psxCpu->Execute();
@@ -529,26 +627,40 @@ LRESULT WINAPI MainWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {
 				case ID_EMULATOR_RESET:
 					SetMenu(hWnd, NULL);
 					OpenPlugins(hWnd);
-					SysReset();
 					CheckCdrom();
+
+					// Auto-detect: region first, then rcnt reset
+					SysReset();
+					
 					LoadCdrom();
-					ShowCursor(FALSE);
+					if(Config.HideCursor)
+						ShowCursor(FALSE);
 					Running = 1;
 					psxCpu->Execute();
 					return TRUE;
 
+				case ID_EMULATOR_SHUTDOWN:
+					ReleasePlugins();
+					SetIsoFile(NULL);
+					CdromId[0] = '\0';
+					CdromLabel[0] = '\0';
+					UpdateMenuItems();
+					ShowCursor(TRUE); // we want GUI to have cursor always
+					return TRUE;
+
 				case ID_EMULATOR_SWITCH_ISO:
 					if (!Open_Iso_Proc(File)) return TRUE;
-					strcpy(cdrfilename, File);
+					SetIsoFile(File);
 					SetMenu(hWnd, NULL);
 					if (OpenPlugins(hWnd) == -1) {
 						ClosePlugins();
 						RestoreWindow();
 						return TRUE;
 					}
-					ShowCursor(FALSE);
+					if(Config.HideCursor)
+						ShowCursor(FALSE);
 					Running = 1;
-					cdOpenCase = time(NULL) + 2;
+					SetCdOpenCaseTime(time(NULL) + 2);
 					CheatSearchBackupMemory();
 					psxCpu->Execute();
 					return TRUE;
@@ -570,6 +682,12 @@ LRESULT WINAPI MainWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {
 				    if (CDR_configure) CDR_configure();
 					return TRUE;
 
+				case ID_CONFIGURATION_LINKCABLE:
+#ifdef ENABLE_SIO1API
+					if (SIO1_configure) SIO1_configure();
+#endif
+					return TRUE;
+
 				case ID_CONFIGURATION_NETPLAY:
 					DialogBox(gApp.hInstance, MAKEINTRESOURCE(IDD_NETPLAY), hWnd, (DLGPROC)ConfigureNetPlayDlgProc);
 					return TRUE;
@@ -840,14 +958,22 @@ void UpdateMcdItems(int mcd, int idc) {
 void McdListGetDispInfo(int mcd, int idc, LPNMHDR pnmh) {
 	LV_DISPINFO *lpdi = (LV_DISPINFO *)pnmh;
 	McdBlock *Info;
+	char buf[256];
+	static char buftitle[256];
 
-	Info = &Blocks[mcd-1][lpdi->item.iItem];
+	Info = &Blocks[mcd - 1][lpdi->item.iItem];
 
 	switch (lpdi->item.iSubItem) {
 		case 0:
 			switch (Info->Flags & 0xF) {
 				case 1:
-					lpdi->item.pszText = Info->Title;
+					if (MultiByteToWideChar(932, 0, (LPCSTR)Info->sTitle, -1, (LPWSTR)buf, sizeof(buf)) == 0) {
+						lpdi->item.pszText = Info->Title;
+					} else if (WideCharToMultiByte(CP_ACP, 0, (LPCWSTR)buf, -1, (LPSTR)buftitle, sizeof(buftitle), NULL, NULL) == 0) {
+						lpdi->item.pszText = Info->Title;
+					} else {
+						lpdi->item.pszText = buftitle;
+					}
 					break;
 				case 2:
 					lpdi->item.pszText = _("mid link block");
@@ -1203,49 +1329,66 @@ BOOL CALLBACK ConfigureCpuDlgProc(HWND hW, UINT uMsg, WPARAM wParam, LPARAM lPar
 			Button_SetText(GetDlgItem(hW,IDC_SIO),     _("Sio Irq Always Enabled"));
 			Button_SetText(GetDlgItem(hW,IDC_MDEC),    _("Black && White Movies"));
 			Button_SetText(GetDlgItem(hW,IDC_CDDA),    _("Disable Cd audio"));
+			Button_SetText(GetDlgItem(hW,IDC_SLOWBOOT),_("Slow Boot"));
 			Button_SetText(GetDlgItem(hW,IDC_PSXAUTO), _("Autodetect"));
 			Button_SetText(GetDlgItem(hW,IDC_CPU),     _("Enable Interpreter Cpu"));
 			Button_SetText(GetDlgItem(hW,IDC_PSXOUT),  _("Enable Console Output"));
+			Button_SetText(GetDlgItem(hW,IDC_DEBUG),   _("Enable Debugger"));
 			Button_SetText(GetDlgItem(hW,IDC_SPUIRQ),  _("Spu Irq Always Enabled"));
 			Button_SetText(GetDlgItem(hW,IDC_RCNTFIX), _("Parasite Eve 2, Vandal Hearts 1/2 Fix"));
 			Button_SetText(GetDlgItem(hW,IDC_VSYNCWA), _("InuYasha Sengoku Battle Fix"));
+			Button_SetText(GetDlgItem(hW,IDC_WIDESCREEN), _("Widescreen (GTE Hack)"));
+			Button_SetText(GetDlgItem(hW,IDC_HIDECURSOR), _("Hide cursor"));
 
 			Static_SetText(GetDlgItem(hW,IDC_MISCOPT), _("Options"));
 			Static_SetText(GetDlgItem(hW,IDC_SELPSX),  _("Psx System Type"));
 
 			Button_SetCheck(GetDlgItem(hW,IDC_XA),      Config.Xa);
-			Button_SetCheck(GetDlgItem(hW,IDC_SIO),     Config.Sio);
+			Button_SetCheck(GetDlgItem(hW,IDC_SIO),     Config.SioIrq);
 			Button_SetCheck(GetDlgItem(hW,IDC_MDEC),    Config.Mdec);
 			Button_SetCheck(GetDlgItem(hW,IDC_CDDA),    Config.Cdda);
+			Button_SetCheck(GetDlgItem(hW,IDC_SLOWBOOT),Config.SlowBoot);
 	   		Button_SetCheck(GetDlgItem(hW,IDC_PSXAUTO), Config.PsxAuto);
-	   		Button_SetCheck(GetDlgItem(hW,IDC_CPU),     Config.Cpu);
+	   		Button_SetCheck(GetDlgItem(hW,IDC_CPU),     (Config.Cpu == CPU_INTERPRETER));
 	   		Button_SetCheck(GetDlgItem(hW,IDC_PSXOUT),  Config.PsxOut);
+			Button_SetCheck(GetDlgItem(hW,IDC_DEBUG),   Config.Debug);
 	   		Button_SetCheck(GetDlgItem(hW,IDC_SPUIRQ),  Config.SpuIrq);
 	   		Button_SetCheck(GetDlgItem(hW,IDC_RCNTFIX), Config.RCntFix);
 	   		Button_SetCheck(GetDlgItem(hW,IDC_VSYNCWA), Config.VSyncWA);
-	   		ComboBox_AddString(GetDlgItem(hW,IDC_PSXTYPES),"NTSC");
-	   		ComboBox_AddString(GetDlgItem(hW,IDC_PSXTYPES),"PAL");
+			Button_SetCheck(GetDlgItem(hW,IDC_WIDESCREEN), Config.Widescreen);
+			Button_SetCheck(GetDlgItem(hW,IDC_HIDECURSOR), Config.HideCursor);
+	   		ComboBox_AddString(GetDlgItem(hW,IDC_PSXTYPES), "NTSC");
+	   		ComboBox_AddString(GetDlgItem(hW,IDC_PSXTYPES), "PAL");
 	   		ComboBox_SetCurSel(GetDlgItem(hW,IDC_PSXTYPES),Config.PsxType);
 
+			if (Config.Cpu == CPU_DYNAREC) {
+				Config.Debug = 0;
+				Button_SetCheck(GetDlgItem(hW, IDC_DEBUG), FALSE);
+				EnableWindow(GetDlgItem(hW, IDC_DEBUG), FALSE);
+			}
+
+			EnableWindow(GetDlgItem(hW,IDC_PSXTYPES), !Config.PsxAuto);
+			break;
+
 		case WM_COMMAND: {
-     		switch (LOWORD(wParam)) {
-       			case IDCANCEL: EndDialog(hW,FALSE); return TRUE;
+			switch (LOWORD(wParam)) {
+       			case IDCANCEL: EndDialog(hW, FALSE); return TRUE;
         		case IDOK:
 					tmp = ComboBox_GetCurSel(GetDlgItem(hW,IDC_PSXTYPES));
 					if (tmp == 0) Config.PsxType = 0;
 					else Config.PsxType = 1;
 
 					Config.Xa      = Button_GetCheck(GetDlgItem(hW,IDC_XA));
-					Config.Sio     = Button_GetCheck(GetDlgItem(hW,IDC_SIO));
+					Config.SioIrq  = Button_GetCheck(GetDlgItem(hW,IDC_SIO));
 					Config.Mdec    = Button_GetCheck(GetDlgItem(hW,IDC_MDEC));
 					Config.Cdda    = Button_GetCheck(GetDlgItem(hW,IDC_CDDA));
+					Config.SlowBoot= Button_GetCheck(GetDlgItem(hW,IDC_SLOWBOOT));
 					Config.PsxAuto = Button_GetCheck(GetDlgItem(hW,IDC_PSXAUTO));
 					tmp = Config.Cpu;
-					Config.Cpu     = Button_GetCheck(GetDlgItem(hW,IDC_CPU));
+					Config.Cpu     = (Button_GetCheck(GetDlgItem(hW,IDC_CPU)) ? CPU_INTERPRETER : CPU_DYNAREC);
 					if (tmp != Config.Cpu) {
 						psxCpu->Shutdown();
-						if (Config.Cpu)	
-							 psxCpu = &psxInt;
+						if (Config.Cpu == CPU_INTERPRETER) psxCpu = &psxInt;
 						else psxCpu = &psxRec;
 						if (psxCpu->Init() == -1) {
 							SysClose();
@@ -1257,6 +1400,14 @@ BOOL CALLBACK ConfigureCpuDlgProc(HWND hW, UINT uMsg, WPARAM wParam, LPARAM lPar
 					Config.SpuIrq  = Button_GetCheck(GetDlgItem(hW,IDC_SPUIRQ));
 					Config.RCntFix = Button_GetCheck(GetDlgItem(hW,IDC_RCNTFIX));
 					Config.VSyncWA = Button_GetCheck(GetDlgItem(hW,IDC_VSYNCWA));
+					Config.Widescreen = Button_GetCheck(GetDlgItem(hW,IDC_WIDESCREEN));
+					Config.HideCursor = Button_GetCheck(GetDlgItem(hW,IDC_HIDECURSOR));
+					tmp = Config.Debug;
+					Config.Debug   = Button_GetCheck(GetDlgItem(hW,IDC_DEBUG));
+					if (tmp != Config.Debug) {
+						if (Config.Debug) StartDebugger();
+						else StopDebugger();
+					}
 
 					SaveConfig();
 
@@ -1266,6 +1417,23 @@ BOOL CALLBACK ConfigureCpuDlgProc(HWND hW, UINT uMsg, WPARAM wParam, LPARAM lPar
 					else CloseConsole();
 
 					return TRUE;
+
+				case IDC_CPU:
+					if (Button_GetCheck(GetDlgItem(hW,IDC_CPU))) {
+						EnableWindow(GetDlgItem(hW,IDC_DEBUG), TRUE);
+					} else {
+						Button_SetCheck(GetDlgItem(hW,IDC_DEBUG), FALSE);
+						EnableWindow(GetDlgItem(hW,IDC_DEBUG), FALSE);
+					}
+					break;
+
+				case IDC_PSXAUTO:
+					if (Button_GetCheck(GetDlgItem(hW,IDC_PSXAUTO))) {
+						EnableWindow(GetDlgItem(hW,IDC_PSXTYPES), FALSE);
+					} else {
+						EnableWindow(GetDlgItem(hW,IDC_PSXTYPES), TRUE);
+					}
+					break;
 			}
 		}
 	}
@@ -1389,9 +1557,9 @@ int Open_Iso_Proc(char *file) {
     ofn.lStructSize			= sizeof(OPENFILENAME);
     ofn.hwndOwner			= gApp.hWnd;
 
-	strcpy(szFilter, _("Psx Isos (*.iso;*.mdf;*.img;*.bin)"));
+	strcpy(szFilter, _("Psx Isos (*.iso;*.mdf;*.img;*.bin;*.cue;*.pbp;*.cbn)"));
 	str = szFilter + strlen(szFilter) + 1; 
-	strcpy(str, "*.iso;*.mdf;*.img;*.bin");
+	strcpy(str, "*.iso;*.mdf;*.img;*.bin;*.cue;*.pbp;*.cbn");
 
 	str += strlen(str) + 1;
 	strcpy(str, _("All Files"));
@@ -1468,33 +1636,42 @@ void CreateMainMenu() {
 	ADDSUBMENU(0, _("&File"));
 	ADDMENUITEM(0, _("E&xit"), ID_FILE_EXIT);
 	ADDSEPARATOR(0);
-	ADDSUBMENUS(0, 1, _("&States"));
-	ADDSEPARATOR(0);
 	ADDMENUITEM(0, _("Run &EXE..."), ID_FILE_RUN_EXE);
 	ADDMENUITEM(0, _("Run &BIOS"), ID_FILE_RUNBIOS);
 	ADDMENUITEM(0, _("Run &ISO..."), ID_FILE_RUN_ISO);
 	ADDMENUITEM(0, _("Run &CD"), ID_FILE_RUN_CD);
+
+	ADDSUBMENU(0, _("&Emulator"));
+	ADDSUBMENUS(0, 1, _("&States"));
+	ADDSEPARATOR(0);
+	ADDMENUITEM(0, _("S&witch ISO..."), ID_EMULATOR_SWITCH_ISO);
+	ADDSEPARATOR(0);
+	ADDMENUITEM(0, _("S&hutdown"), ID_EMULATOR_SHUTDOWN);
+	ADDMENUITEM(0, _("Re&set"), ID_EMULATOR_RESET);
+	ADDMENUITEM(0, _("&Run"), ID_EMULATOR_RUN);
 	ADDSUBMENUS(1, 3, _("&Save"));
 	ADDSUBMENUS(1, 2, _("&Load"));
 	ADDMENUITEM(2, _("&Other..."), ID_FILE_STATES_LOAD_OTHER);
+	ADDMENUITEM(2, _("Slot &9"), ID_FILE_STATES_LOAD_SLOT9);
+	ADDMENUITEM(2, _("Slot &8"), ID_FILE_STATES_LOAD_SLOT8);
+	ADDMENUITEM(2, _("Slot &7"), ID_FILE_STATES_LOAD_SLOT7);
+	ADDMENUITEM(2, _("Slot &6"), ID_FILE_STATES_LOAD_SLOT6);
 	ADDMENUITEM(2, _("Slot &5"), ID_FILE_STATES_LOAD_SLOT5);
 	ADDMENUITEM(2, _("Slot &4"), ID_FILE_STATES_LOAD_SLOT4);
 	ADDMENUITEM(2, _("Slot &3"), ID_FILE_STATES_LOAD_SLOT3);
 	ADDMENUITEM(2, _("Slot &2"), ID_FILE_STATES_LOAD_SLOT2);
 	ADDMENUITEM(2, _("Slot &1"), ID_FILE_STATES_LOAD_SLOT1);
 	ADDMENUITEM(3, _("&Other..."), ID_FILE_STATES_SAVE_OTHER);
+	ADDMENUITEM(3, _("Slot &9"), ID_FILE_STATES_SAVE_SLOT9);
+	ADDMENUITEM(3, _("Slot &8"), ID_FILE_STATES_SAVE_SLOT8);
+	ADDMENUITEM(3, _("Slot &7"), ID_FILE_STATES_SAVE_SLOT7);
+	ADDMENUITEM(3, _("Slot &6"), ID_FILE_STATES_SAVE_SLOT6);
 	ADDMENUITEM(3, _("Slot &5"), ID_FILE_STATES_SAVE_SLOT5);
 	ADDMENUITEM(3, _("Slot &4"), ID_FILE_STATES_SAVE_SLOT4);
 	ADDMENUITEM(3, _("Slot &3"), ID_FILE_STATES_SAVE_SLOT3);
 	ADDMENUITEM(3, _("Slot &2"), ID_FILE_STATES_SAVE_SLOT2);
 	ADDMENUITEM(3, _("Slot &1"), ID_FILE_STATES_SAVE_SLOT1);
 
-	ADDSUBMENU(0, _("&Emulator"));
-	ADDMENUITEM(0, _("S&witch ISO..."), ID_EMULATOR_SWITCH_ISO);
-	ADDSEPARATOR(0);
-	ADDMENUITEM(0, _("Re&set"), ID_EMULATOR_RESET);
-	ADDMENUITEM(0, _("&Run"), ID_EMULATOR_RUN);
-
 	ADDSUBMENU(0, _("&Configuration"));
 	ADDMENUITEM(0, _("Cheat &Search..."), ID_CONFIGURATION_CHEATSEARCH);
 	ADDMENUITEM(0, _("Ch&eat Code..."), ID_CONFIGURATION_CHEATLIST);
@@ -1502,6 +1679,7 @@ void CreateMainMenu() {
 #ifdef ENABLE_NLS
 	ADDSUBMENUS(0, 1, _("&Language"));
 
+	// wtf?
 	if (langs != langs) free(langs);
 	langs = (_langs*)malloc(sizeof(_langs));
 	strcpy(langs[0].lang, "English");
@@ -1530,6 +1708,7 @@ void CreateMainMenu() {
 	ADDSEPARATOR(0);
 	ADDMENUITEM(0, _("&NetPlay..."), ID_CONFIGURATION_NETPLAY);
 	ADDSEPARATOR(0);
+	ADDMENUITEM(0, _("&Link cable..."), ID_CONFIGURATION_LINKCABLE);
 	ADDMENUITEM(0, _("&Controllers..."), ID_CONFIGURATION_CONTROLLERS);
 	ADDMENUITEM(0, _("CD-&ROM..."), ID_CONFIGURATION_CDROM);
 	ADDMENUITEM(0, _("&Sound..."), ID_CONFIGURATION_SOUND);
@@ -1540,41 +1719,14 @@ void CreateMainMenu() {
 	ADDSUBMENU(0, _("&Help"));
 	ADDMENUITEM(0, _("&About..."), ID_HELP_ABOUT);
 
-	if (CdromId[0] != '\0') {
-		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION_NETPLAY, MF_BYCOMMAND | MF_GRAYED);
-		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION_CONTROLLERS, MF_BYCOMMAND | MF_GRAYED);
-		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION_CDROM, MF_BYCOMMAND | MF_GRAYED);
-		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION_SOUND, MF_BYCOMMAND | MF_GRAYED);
-		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION_GRAPHICS, MF_BYCOMMAND | MF_GRAYED);
-		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION, MF_BYCOMMAND | MF_GRAYED);
-		if (cdrfilename[0] == '\0') {
-			EnableMenuItem(gApp.hMenu, ID_EMULATOR_SWITCH_ISO, MF_BYCOMMAND | MF_GRAYED);
-		}
-	} else {
-		EnableMenuItem(gApp.hMenu, ID_EMULATOR_RESET, MF_BYCOMMAND | MF_GRAYED);
-		EnableMenuItem(gApp.hMenu, ID_EMULATOR_RUN, MF_BYCOMMAND | MF_GRAYED);
-		EnableMenuItem(gApp.hMenu, ID_EMULATOR_SWITCH_ISO, MF_BYCOMMAND | MF_GRAYED);
-		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_LOAD_SLOT1, MF_BYCOMMAND | MF_GRAYED);
-		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_LOAD_SLOT2, MF_BYCOMMAND | MF_GRAYED);
-		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_LOAD_SLOT3, MF_BYCOMMAND | MF_GRAYED);
-		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_LOAD_SLOT4, MF_BYCOMMAND | MF_GRAYED);
-		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_LOAD_SLOT5, MF_BYCOMMAND | MF_GRAYED);
-		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_LOAD_OTHER, MF_BYCOMMAND | MF_GRAYED);
-		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_SAVE_SLOT1, MF_BYCOMMAND | MF_GRAYED);
-		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_SAVE_SLOT2, MF_BYCOMMAND | MF_GRAYED);
-		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_SAVE_SLOT3, MF_BYCOMMAND | MF_GRAYED);
-		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_SAVE_SLOT4, MF_BYCOMMAND | MF_GRAYED);
-		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_SAVE_SLOT5, MF_BYCOMMAND | MF_GRAYED);
-		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_SAVE_OTHER, MF_BYCOMMAND | MF_GRAYED);
-		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION_CHEATSEARCH, MF_BYCOMMAND | MF_GRAYED);
-	}
+	UpdateMenuItems();
 }
 
 void CreateMainWindow(int nCmdShow) {
 	WNDCLASS wc;
 	HWND hWnd;
 
-	wc.lpszClassName = "PCSX Main";
+	wc.lpszClassName = "PCSXR Main";
 	wc.lpfnWndProc = MainWndProc;
 	wc.style = 0;
 	wc.hInstance = gApp.hInstance;
@@ -1587,20 +1739,19 @@ void CreateMainWindow(int nCmdShow) {
 
 	RegisterClass(&wc);
 
-	hWnd = CreateWindow("PCSX Main",
-						"PCSX",
+	hWnd = CreateWindow("PCSXR Main",
+						"PCSXR",
 						WS_CAPTION | WS_POPUPWINDOW | WS_MINIMIZEBOX,
 						CW_USEDEFAULT,
 						0,
-						350,
-						220,
+						356,
+						252,
 						NULL,
 						NULL,
 						gApp.hInstance,
 						NULL);
 
 	gApp.hWnd = hWnd;
-	ResetMenuSlots();
 
 	CreateMainMenu();
 	SetMenu(gApp.hWnd, gApp.hMenu);
@@ -1657,11 +1808,7 @@ void ChangeLanguage(char *lang) {
 int SysInit() {
 	if (Config.PsxOut) OpenConsole();
 
-	if (psxInit() == -1) return -1;
-
-#ifdef GTE_DUMP
-	gteLog = fopen("gteLog.txt","w");
-#endif
+	if (EmuInit() == -1) return -1;
 
 #ifdef EMU_LOG
 	emuLog = fopen("emuLog.txt","w");
@@ -1675,26 +1822,27 @@ int SysInit() {
 	}
 	LoadMcds(Config.Mcd1, Config.Mcd2);
 
+	if (Config.Debug) StartDebugger();
+
 	return 0;
 }
 
 void SysReset() {
-	psxReset();
+	EmuReset();
 }
 
 void SysClose() {
-	psxShutdown();
+	EmuShutdown();
 	ReleasePlugins();
 
+	StopDebugger();
+
 	if (Config.PsxOut) CloseConsole();
 
 	if (emuLog != NULL) fclose(emuLog);
-#ifdef GTE_DUMP
-	if (gteLog != NULL) fclose(gteLog);
-#endif
 }
 
-void SysPrintf(char *fmt, ...) {
+void SysPrintf(const char *fmt, ...) {
 	va_list list;
 	char msg[512];
 	DWORD tmp;
@@ -1713,24 +1861,24 @@ void SysPrintf(char *fmt, ...) {
 #endif
 }
 
-void SysMessage(char *fmt, ...) {
+void SysMessage(const char *fmt, ...) {
 	va_list list;
 	char tmp[512];
 
 	va_start(list,fmt);
 	vsprintf(tmp,fmt,list);
 	va_end(list);
-	MessageBox(0, tmp, _("Pcsx Msg"), 0);
+	MessageBox(0, tmp, _("Pcsxr Msg"), 0);
 }
 
 static char *err = N_("Error Loading Symbol");
 static int errval;
 
-void *SysLoadLibrary(char *lib) {
+void *SysLoadLibrary(const char *lib) {
 	return LoadLibrary(lib);
 }
 
-void *SysLoadSym(void *lib, char *sym) {
+void *SysLoadSym(void *lib, const char *sym) {
 	void *tmp = GetProcAddress((HINSTANCE)lib, sym);
 	if (tmp == NULL) errval = 1;
 	else errval = 0;
@@ -1759,3 +1907,57 @@ void SysRunGui() {
 	RestoreWindow();
 	RunGui();
 }
+
+void UpdateMenuItems() {
+	if (CdromId[0] != '\0') { // Emulating...
+		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION_NETPLAY, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION_LINKCABLE, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION_CONTROLLERS, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION_CDROM, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION_SOUND, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION_GRAPHICS, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION, MF_BYCOMMAND | MF_GRAYED);
+		if (!UsingIso()) {
+			EnableMenuItem(gApp.hMenu, ID_EMULATOR_SWITCH_ISO, MF_BYCOMMAND | MF_GRAYED);
+		}
+
+		ResetMenuSlots();
+	} else { // GUI...
+		EnableMenuItem(gApp.hMenu, ID_EMULATOR_RESET, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_EMULATOR_RUN, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_EMULATOR_SHUTDOWN, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_EMULATOR_SWITCH_ISO, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_LOAD_SLOT1, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_LOAD_SLOT2, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_LOAD_SLOT3, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_LOAD_SLOT4, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_LOAD_SLOT5, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_LOAD_SLOT6, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_LOAD_SLOT7, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_LOAD_SLOT8, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_LOAD_SLOT9, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_LOAD_OTHER, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_SAVE_SLOT1, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_SAVE_SLOT2, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_SAVE_SLOT3, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_SAVE_SLOT4, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_SAVE_SLOT5, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_SAVE_SLOT6, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_SAVE_SLOT7, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_SAVE_SLOT8, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_SAVE_SLOT9, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_FILE_STATES_SAVE_OTHER, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION_CHEATSEARCH, MF_BYCOMMAND | MF_GRAYED);
+		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION_NETPLAY, MF_BYCOMMAND | MF_ENABLED);
+		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION_LINKCABLE, MF_BYCOMMAND | MF_ENABLED);
+		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION_CONTROLLERS, MF_BYCOMMAND | MF_ENABLED);
+		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION_CDROM, MF_BYCOMMAND | ( Config.Cdr[0] != '\0' ? MF_ENABLED : MF_GRAYED ));
+		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION_SOUND, MF_BYCOMMAND | MF_ENABLED);
+		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION_GRAPHICS, MF_BYCOMMAND | MF_ENABLED);
+		EnableMenuItem(gApp.hMenu, ID_CONFIGURATION, MF_BYCOMMAND | MF_ENABLED);
+	}
+
+#ifndef ENABLE_SIO1API
+	EnableMenuItem(gApp.hMenu, ID_CONFIGURATION_LINKCABLE, MF_BYCOMMAND | MF_GRAYED);
+#endif
+}
